package jif.util;

import jif.runtime.Runtime;

final public class Date implements jif.lang.JifObject {
    private int year;
    private int month;
    private int day;
    final public static String[] monthNames = { "NOT A VALID MONTH", "January",
    "February", "March", "April", "May", "June", "July", "August", "September",
    "October", "November", "December" };
    final public static String[] daysOfWeek = { "Sunday", "Monday", "Tuesday",
    "Wednesday", "Thursday", "Friday", "Saturday", "Sunday" };
    final public static String[] shortDaysOfWeek = { "Sun", "Mon", "Tue", "Wed",
    "Thu", "Fri", "Sat", "Sun" };
    final public static String[] shortMonthNames = { "NOT A VALID MONTH", "Jan",
    "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov",
    "Dec" };
    final public static int[] maxDaysInMonth = { -1, 31, 29, 31, 30, 31, 30, 31,
    31, 30, 31, 30, 31 };
    
    public static Date valueOf(final jif.lang.Label jif$L,
                               final jif.lang.Label lbl, final String d)
          throws IllegalArgumentException {
        if (d == null) { throw new IllegalArgumentException("null string"); }
        int yearStart = 0;
        int yearEnd = 4;
        int monthStart = 5;
        int monthEnd = 7;
        int dayStart = 8;
        int dayEnd = 10;
        int i = d.indexOf('-', 0);
        if (i > 0) {
            dayStart = 0;
            dayEnd = i;
            monthStart = i + 1;
            int j = d.indexOf('-', monthStart);
            if (j > 0) {
                monthEnd = j;
                yearStart = j + 1;
                yearEnd = d.length();
            } else {
                throw new IllegalArgumentException(
                  "invalid string: format should be DD-MM-YYYY");
            }
        } else
                  if (d.length() != 10) {
                      throw new IllegalArgumentException(
                        "invalid string: format should be DD-MM-YYYY");
                  }
        try {
            int year = Integer.parseInt(d.substring(yearStart, yearEnd));
            int month = Integer.parseInt(d.substring(monthStart, monthEnd));
            int day = Integer.parseInt(d.substring(dayStart, dayEnd));
            return new Date(lbl).jif$util$Date$(year, month, day);
        }
        catch (final NumberFormatException e) {
            throw new IllegalArgumentException(
              "invalid string: format should be DD-MM-YYYY");
        }
        catch (final StringIndexOutOfBoundsException impossible) {
            return null;
        }
    }
    
    public Date jif$util$Date$() {
        this.jif$init();
        {
            final jif.lang.Principal p = jif.lang.PrincipalUtil.nullPrincipal();
            this.year = Runtime.currentYear(p);
            this.month = Runtime.currentMonth(p);
            this.day = Runtime.currentDayOfMonth(p);
        }
        return this;
    }
    
    public Date jif$util$Date$(final int pYear, final int pMonth,
                               final int pDay)
          throws IllegalArgumentException {
        this.jif$init();
        {
            this.year = pYear;
            this.month = pMonth;
            this.day = pDay;
            checkDateValid(this.jif$jif_util_Date_L, pYear, pMonth, pDay);
        }
        return this;
    }
    
    public Date jif$util$Date$(final int pYear, final String pMonth,
                               final int pDay)
          throws IllegalArgumentException {
        this.jif$init();
        {
            this.year = pYear;
            this.day = pDay;
            int theMonth = 0;
            for (int i = 1; i <= 12; i++) {
                try {
                    if (monthNames[i].equalsIgnoreCase(pMonth) ||
                          shortMonthNames[i].equalsIgnoreCase(pMonth)) {
                        theMonth = i;
                        break;
                    }
                }
                catch (final ArrayIndexOutOfBoundsException impossible) {  }
                catch (final NullPointerException impossible) {  }
            }
            if (theMonth == 0) {
                throw new IllegalArgumentException("Invalid month: " + pMonth);
            }
            this.month = theMonth;
            checkDateValid(this.jif$jif_util_Date_L, pYear, theMonth, pDay);
        }
        return this;
    }
    
    public static int daysInMonth(final jif.lang.Label jif$L, final int month,
                                  final int year) {
        if (month < 1 || month > 12) return -1;
        try {
            if (month == 2) { return isLeapYear(jif$L, year) ? 29 : 28; }
            return maxDaysInMonth[month];
        }
        catch (final NullPointerException impossible) {  }
        catch (final ArrayIndexOutOfBoundsException impossible) {  }
        return -1;
    }
    
    private static void checkDateValid(final jif.lang.Label jif$L,
                                       final int pYear, final int pMonth,
                                       final int pDay)
          throws IllegalArgumentException {
        if (pMonth < 1 || pMonth > 12) {
            throw new IllegalArgumentException("Invalid month: " + pMonth);
        }
        try {
            if (pDay < 1 || pDay > daysInMonth(jif$L, pMonth, pYear)) {
                throw new IllegalArgumentException(monthNames[pMonth] +
                                                   " " +
                                                   pYear +
                                                   " has " +
                                                   daysInMonth(jif$L, pMonth,
                                                               pYear) +
                                                   " days.");
            }
        }
        catch (final ArrayIndexOutOfBoundsException impossible) {  }
        catch (final NullPointerException impossible) {  }
    }
    
    public static Date lenientDate(final jif.lang.Label jif$L,
                                   final jif.lang.Label lbl, final int year,
                                   final int month, final int day) {
        int y = year;
        int m = month;
        int d = day;
        try {
            while (m > 12 || m < 1 || d < 1 || d > daysInMonth(jif$L, m, y)) {
                while (m > 12) {
                    m -= 12;
                    y++;
                }
                while (m < 1) {
                    m += 12;
                    y--;
                }
                if (d > daysInMonth(jif$L, m, y)) {
                    d -= daysInMonth(jif$L, m, y);
                    m++;
                }
                if (d < 1) {
                    d += daysInMonth(jif$L, (m + 10) % 12 + 1, y);
                    m--;
                }
            }
            return new Date(lbl).jif$util$Date$(y, m, d);
        }
        catch (final ArithmeticException impossible) {  }
        catch (final IllegalArgumentException impossible) {  }
        return null;
    }
    
    public int getYear() { return this.year; }
    
    public int getMonth() { return this.month; }
    
    public int getDay() { return this.day; }
    
    public int getDayOfWeek() {
        try {
            int thisD =
              Date.daysAfterJan_1_1900(this.jif$jif_util_Date_L, this.year,
                                       this.month, this.day);
            return (thisD % 7 + 8) % 7;
        }
        catch (final ArithmeticException unlikely) { return 0; }
    }
    
    public String getMonthName() {
        try {
            return monthNames[this.month];
        }
        catch (final ArrayIndexOutOfBoundsException impossible) {  }
        catch (final NullPointerException impossible) {  }
        return "impossible!";
    }
    
    public String getShortMonthName() {
        try {
            return monthNames[this.month];
        }
        catch (final ArrayIndexOutOfBoundsException impossible) {  }
        catch (final NullPointerException impossible) {  }
        return "impossible!";
    }
    
    public String getDayOfWeekName() {
        try {
            return daysOfWeek[this.getDayOfWeek()];
        }
        catch (final ArrayIndexOutOfBoundsException impossible) {  }
        catch (final NullPointerException impossible) {  }
        return "impossible!";
    }
    
    public String getShortDayOfWeekName() {
        try {
            return shortDaysOfWeek[this.getDayOfWeek()];
        }
        catch (final ArrayIndexOutOfBoundsException impossible) {  }
        catch (final NullPointerException impossible) {  }
        return "impossible!";
    }
    
    public boolean before(final Date when) {
        Date that = when;
        if (that == null) { return false; }
        return this.year < that.year || this.year == that.year && this.month <
          that.month || this.year == that.year && this.month == that.month &&
          this.day < that.day;
    }
    
    public boolean after(final Date when) {
        Date that = when;
        if (that == null) { return false; }
        return this.year > that.year || this.year == that.year && this.month >
          that.month || this.year == that.year && this.month == that.month &&
          this.day > that.day;
    }
    
    public int daysBefore(final Date when) {
        if (when == null) return 0;
        int whenD =
          Date.daysAfterJan_1_1900(this.jif$jif_util_Date_L, when.year,
                                   when.month, when.day);
        int thisD =
          Date.daysAfterJan_1_1900(this.jif$jif_util_Date_L, this.year,
                                   this.month, this.day);
        return whenD - thisD;
    }
    
    private static int daysAfterJan_1_1900(final jif.lang.Label jif$L,
                                           final int year, final int month,
                                           final int day) {
        int y = year;
        int m = month;
        int d = day;
        int count = 0;
        while (y - 400 > 1900) {
            count += DAYS_IN_400_YEARS;
            y -= 400;
        }
        while (y + 400 < 1900) {
            count -= DAYS_IN_400_YEARS;
            y += 400;
        }
        while (y > 1900) { count += (isLeapYear(jif$L, --y) ? 366 : 365); }
        while (y < 1900) { count -= (isLeapYear(jif$L, ++y) ? 366 : 365); }
        while (m > 1) { count += daysInMonth(jif$L, --m, y); }
        count += d - 1;
        return count;
    }
    
    final private static int DAYS_IN_400_YEARS = 146097;
    
    public static boolean isLeapYear(final jif.lang.Label jif$L,
                                     final int pYear) {
        try {
            return pYear % 4 == 0 && (pYear % 100 != 0 || pYear % 400 == 0);
        }
        catch (final ArithmeticException e) { return false; }
    }
    
    public String toString() { return toLongString(); }
    
    public boolean equals(final jif.lang.IDComparable o) {
        return equals(this.jif$jif_util_Date_L, o);
    }
    
    public boolean equals(final jif.lang.Label lbl,
                          final jif.lang.IDComparable o) {
        if (o == null || !Date.jif$Instanceof(lbl, o)) { return false; }
        Date that = Date.jif$cast$jif_util_Date(lbl, o);
        return this.year == that.year && this.month == that.month && this.day ==
          that.day;
    }
    
    public int hashCode() { return year ^ month * 37 & day * 113; }
    
    public String toLongString() {
        return day + " " + getMonthName() + " " + year;
    }
    
    public String toShortString() { return day + "-" + month + "-" + year; }
    
    public Date year(final int year) throws IllegalArgumentException {
        return new Date(this.jif$jif_util_Date_L).jif$util$Date$(year,
                                                                 this.month,
                                                                 this.day);
    }
    
    public Date month(final int month) throws IllegalArgumentException {
        return new Date(this.jif$jif_util_Date_L).jif$util$Date$(this.year,
                                                                 month,
                                                                 this.day);
    }
    
    public Date day(final int day) throws IllegalArgumentException {
        return new Date(this.jif$jif_util_Date_L).jif$util$Date$(this.year,
                                                                 this.month,
                                                                 day);
    }
    
    public static Date addYear(final jif.lang.Label jif$L,
                               final jif.lang.Label lb, final Date date,
                               final int delta) {
        if (date == null) return null;
        return Date.lenientDate(lb, lb, date.getYear() + delta, date.getMonth(),
                                date.getDay());
    }
    
    public static Date addMonth(final jif.lang.Label jif$L,
                                final jif.lang.Label lb, final Date date,
                                final int delta) {
        if (date == null) return null;
        return Date.lenientDate(lb, lb, date.getYear(), date.getMonth() +
                                  +delta, date.getDay());
    }
    
    public static Date addDay(final jif.lang.Label jif$L,
                              final jif.lang.Label lb, final Date date,
                              final int delta) {
        if (date == null) return null;
        return Date.lenientDate(lb, lb, date.getYear(), date.getMonth(),
                                date.getDay() + delta);
    }
    
    final public static String jlc$CompilerVersion$jif = "3.0.0";
    final public static long jlc$SourceLastModified$jif = 1149825579000L;
    final public static String jlc$ClassType$jif =
      ("H4sIAAAAAAAAAOV9e3wU1b347Gazu8mCvOVhCOElIAqoPARa2/AyaEISIGAQ" +
       "DZvdSbJms88JBloV\nRQGLr4tii43V67WtVWpr7aft1dqfVG5BqLXqr9X+qr" +
       "YV6pXb+sD6vLXld77nzJk5c86Z3Zlkl8vP\n3x9zMjn7PY/v+3zPfM/M3reU" +
       "0mxGGXtlrG2GtimlZmdcHGtrCGeyarQhGd+0GlW1RF7Y9o1//Y8L\n3jzgVU" +
       "pqlbJwt9aRzMS0TZoyuPbK8MbwzG4tFp9ZG8tqC2uVwbFEVgsntFhYU6PLMs" +
       "kuTRlfm0Jd\ntceT2ky1R5uZCmfCXTPxYDMbFsfD2Sxq5se12bRyjeLpyShV" +
       "tIU+KTIjDEymdO79s05/dP2PhpQo\ng9Ypg2KJVVpYi0UWJxMaGmKdMqBL7W" +
       "pVM9nqaFSNrlOGJFQ1ukrNxMLx2GYEmEysU4ZmY+2JsNad\nUbMr1WwyvhEA" +
       "h2a7U2oGj0kra5UBkSTCKdMd0ZIZMkM037aYGo/S/0rb4uH2rKacbmJK8FsG" +
       "9Qi9\n8hiaWKYtHFFpE19nLBHVlHF8CwPHSZcgANQ00KUiehtD+RJhVKEMJZ" +
       "SPhxPtM1dpmViiHYGWJrvR\nKJoyxrZTBBRMhSOd4Xa1RVNG8XAN5CcEVYYJ" +
       "AU00ZQQPhntCXBrDcYnhT71/wD++0vBRlRfPOapG\n4jD/AGpUyTVaqbapGT" +
       "URUUnDj7tn3Lm8ubvCqygIeAQHTGCqJ/+4qfbN/zWOwJwhgalvvVKNaC2R\n" +
       "T+dWjH2h+mhZCUwjmEpmY8B8C+ZYeBv0Xxb2pJA2nG70CD/OoD8+tfIXzVse" +
       "Uv/iVfzLFX8kGe/u\nSixXytREdLF+H0D3tbGEulzxxdEfhHlbLK4C5n50nw" +
       "prHfi+J6UoykB0jdT/DtSU4BKkMDOQGmrK\n6His9ZxsJjIT/UemR3/rgeaD" +
       "rvJ40CQreBWJI+mqScajaqYl8u0jh7689JKbdngNkdEH1pSBoOoY\nL+hV8X" +
       "hwbyOtKAMNo6DRf/3BwsG3npP9EVL9dUpZrKurWwu3xhFmA8LxePIqNdqiYR" +
       "kZwsgj1ekB\nrUickGS2xFFHRLNTysaMMoEXG1O9lmPjEVG/NHeFsrty6d3A" +
       "YeDIcOidTA3Rt5PMbcC0VZdfvGHH\nhBIAusqH6FiCQCdYTJmk75bIpn0jFj" +
       "z+1CdPeJXSdchgZZeobeHuuNaweFGyO4GswHCjaqWKDESi\nNtyqxmuVELED" +
       "YaTLVBsDqQhuoykja4FbRD/iAD8Tt0JkCGXMTqDZOCSzk/KToCXSMGzEim9+" +
       "eMbD\nRMx5qjVkkhE1isyX2aBl1vnjV9x33icIL6TBaLYamisYhEpegy1Kt1" +
       "DXUE2ZIBgEfpCF1NgBJj6E\nXFsy0xWOQzeUJuVaRyZ5lVmDhXYgvh+MeBRC" +
       "1xB0ReCCyqFQDCOiDcUk4DeHLLajf1u+veaNw1Mv\n97ImdxDjglapGlHgIa" +
       "a4rM6oKqp/9WsNd+x+a/tlWFaIsHg05HW6W+OxSA+e3EgPks1hEmMyY9Tw\n" +
       "O++a1vsSFcZhZu/VmUx4E8hiz3UvjN1zIHwPMjRI+bOxzSpWcgWPpNABoJyB" +
       "72cyPyIFNMc3pbc6\nm0W8Q3bncyN3vP7c2F+tJuPzrdGEzjAbYdFD7jKWwf" +
       "LWEnlq+NbdN3066CKv4kWsQTLRhvx6LIKc\nd4UgsYuNX0FswWW1U+CxAvBy" +
       "82fwBSP5OejjD1sx4NO/n/HiEjx+KKpmI5lYCrDSraJfS16MyAme\nEI+QCS" +
       "eycWSYiPqsxj8u7UllFhIxgmIy5sJEPCAFN1E2m7RE5mx58/3HfvOjKUSDxl" +
       "lbCNDjHxn7\n7qS9V0ymfK7kUVqphpF1JTijzicd+c7xG4PfwpiVJq/CmjaO" +
       "oVMKueVILBVG7oXewUong3sBRKrR\npEYJvNO7X3hfdzg59pMIzIZB3WrhjC" +
       "FmrE6mjFFaIjV/+vkfbvzqqMMs4lwDBvrcEaNGVb2nlmPF\nMhgykWOI0cCG" +
       "KWhmZ1opzM6JJfTL1415bfrZtx0ks+MZKWvxbw9/9MDmKd9qN1VgiT4o/KmR" +
       "MWst\nWqKazLrhrB2/f/+H51cwzMIcQPhehQEJP6BcZKAzWUa3RUlNS3Yx1D" +
       "vv4KyR1Q/WPUoZtdRoP82K\nGdeSxe/cAff/7M8Pfec+2sdyjFc9g2MjLuel" +
       "dPQ/j8svpsiP9SkWyPpfnf7fkhQllvhfpcU6X4Ku\nGFwy6xyGYqrUnpUYPp" +
       "gxBljL1ChZuD0fOn9H1Zy24ZgLZZhPaGWP1wbj0PoMWtD/y/CQA4yJwVUh\n" +
       "mRgabQzP+upMu259bmoe84P7fj7iLS/4Km8kBoaMXxsno6x3MxdLyFx2p9Ai" +
       "GDt4fYrejTEwnFwX\na8KG1yYascAy7QmSaWvKGuikK5lJdcQiVXjeVcm2Ku" +
       "JOq8KZ9u4uNaFVpZrVcAZ+YGKQqqmtMCU1\nWhVuTW5Uq1o3VX2pdmGV1hHL" +
       "Xj3NEFtD7BaHE4mkJli7/6p/+6nNKfUgFbkLMXc1pRQPKfMqFydj\nCZ2uZ3" +
       "35ho7HlMPrsMVAXqUrlUyo+rJoCkeCYTackxhxYwR2ood/c9uv3vr48a3GRD" +
       "GKhgtGa+84\nWusjj5Kd1JToSkZjbTFYoiKnv2Xsrudv+/qWJmJnzs7fxqwf" +
       "vUjZcviKjyoxgp4IhHbmOsMEI8uN\nkfzqtCac7UDjvxx/ad3uV8+qJOMzix" +
       "L998eX3Lj7rp/8eDZZwA4AcfnCF8GpK16r9SH0X5zcGEYh\nbEJrgFhZ50T1" +
       "RdoPT2/fMBNPdCCOoqksokiC8UYN7E+8mA5H1wipmJ4XoaPqQoqHUJHFrKrF" +
       "cgmL\n/SpLTJEVXC0em+Xpdd9s/+gf+x76gLra0Saulnm2RA4E2oMLHpj7fg" +
       "k2GcyadhSDGyz0mRBXj60r\n7bAnoXWPGEMxitwSueC3G4f4v39vl1cJoCgB" +
       "6x+iwppwvBtkax2K6bOL9UpEecvv1vicBKMLmTj4\nSm4VzRodH0zFtIDmwn" +
       "kUuqbRgJHhkaKk4OYmDHgmLqfp4RDS5rZYIozHnI6Wu1m8S4K46qnNipE4\n" +
       "8kxdKNDdqEfi/1L5wBuPHVk53MtsV0wUAwSmjU5XbLhTYJnH5xoBQ++fPn7v" +
       "NStfayUqMtQahS5N\ndHf956an1Smfu+VPkki2NE5DqkFkQS9dC69GZpF6g2" +
       "u/ed2c7Tf8A7sfbwT7mRuR32nrjsdXGD1j\nzYBiCCLUJMMyax1qVTalRmLI" +
       "PGONQPaiagIY3QmY5nhBO4iPr2FKFZaItDYZCcdNKRt+9aJ5335V\nfZQENX" +
       "E2SOT5w7X8xV3ts+/93vdKqV3cDkW8x0PE4RuiOGB3fQuGJYsIjHA7rujEFb" +
       "frq4I7NKUE\nLf/xD1DcT8ANUZyErlkyZwx/Oixg86RWZa0z51eH1L3Dlfcz" +
       "OBGG6A63x3VpY1JTbOZOSEJc2EZc\nbuZMMtRdbfQzVVfHwphMaHu9qcZfJf" +
       "IH5Z6c0gXVvVA8SBn/EyjuheJhk8nfwl19G/PSvOO4Oh9d\nS/NzFcAuliLe" +
       "5IyrS8Kb+sxTH7TmOPo5m3m75ejn0XXhqcPRJylHnzU4+pTJ0SdwVz81OEru" +
       "5BsN\nHqL+HLtXo+s6O3ab1I3bgOWjbolhPbCw/ZT0O1xD4Susg8AzzliO1l" +
       "Ht4Xi1LhpLeyIq3hswt0Sg\nnCPueUDFPCharfEQqTIDxFZLzCPU1DE19cx9" +
       "o1gDxYuSYAmELwkXQxrMWSjehCJ3sGTlCVynS3rk\nVJAGQzyYpjQUNqpgte" +
       "9NIzawqg1dO4qTKYDaYE14h2rCR4YmvGdqwluY3m8bmvC2zLZV6pY/D2EB\n" +
       "bIYUl8YCeyyOsqyvMkk7GV1nFpW0n+qk9fgM0p4wSfvfmLR/p5o72NRc8swJ" +
       "/wqt/Ry5wcHOzU9u\nAFsoxa++oK6EI7bpRExSz0bXnGKS2hOipB5GSe05zS" +
       "C1B6/2PeVUivU7N/YcXGDGjuamPb/cBsyR\nPX/b4Hg5Fo2XnVvqY6KlPsZY" +
       "6mOCpT4mWOpjjKU+JljqY6al9ozWCQePm/ggbxk8uaXL6K7WL33w\n9DfKq8" +
       "yNxQo88zJh3W5p1hLxPvjH7dNGDfodiiXWKad1hLPLEyiygefLaibfQp7rav" +
       "PPmr7x8bPa\nazgEMENcaD2BrOwN5g3VbdRpcDHM0yXrPHHR74X7WzQlkMrE" +
       "NsICA4n/JmrEVUvHoyQdc1oCvmmk\nBKy/WmIY70EK4IE3JjyzLLMbh65BcI" +
       "loX2iHNvw4F02utMswriLKfKc2KPNgBUZ5qQTlsbqrHyyi\nvDwfyiVR3cSJ" +
       "CPNd2iDMgxUY4QYOYZhZuT70UBHh1SLCJRRhZToUt6IZlmNGQyyf5ZCH7s/W" +
       "iTCU\nR36eAXamDRhvuzzrTVtE7xttauqM3mEFMt1uEkssYFPtwGo44LNy9Q" +
       "lPRvgsDfzMkmzxrH97ZPjJ\n5C2D8fa4rzWcJRtPfHqLmL1iSUrBNipkofSZ" +
       "dhOyMmRSfoZU2oAJDEkzDEkLDEnLGQJGZWJ+hgBY\nlVOGTMjVZw1Mxp9K4e" +
       "XsIHCanstsFAG0f7SoCFucKQKof33bWlXttFGE0Vz3NorAgwl038HQfYdA\n" +
       "9x25FUE6CVERpGB2imAP7OkUhFQKLAppHmJV2oAJxOpliNUrEKs3t5DmIRYV" +
       "UkfEmpCrT4mQbrcR\nUuinUhTShx0J6aBsRzKjLcknqZXcGDaSyoMJxH+MIf" +
       "5jAvEfyy2p0kmIkioFs5NUe2CJpEqBRUnN\nQ6xKGzCBWAcYYh0QiHUgt6Tm" +
       "IRaVVEfEmpCrT4mk/kAiqX6dO5NESf2tC0mty7e4mMSNYSOpPJhA\n/NcY4r" +
       "8mEP+13JIqnYQoqVIwO0m1B5ZIqhRYlNQ8xKq0AROI9R5DrPcEYr2XW1LzEI" +
       "tKqiNiTcjV\np0RSX7WR1HPQNU2QVK/HkaSe1hXuAYu6PFEniXnoM8Rp3BAc" +
       "7SfagPG095aZtKf3jTY1Ju0nKebT\nE2ESSyxgk+3AeNpPydWnTFDH2wHDEt" +
       "g7mqfIGTZNBIpUMBSpEChSIadIhS5meSgCYGOdUmRcrj5B\nGpX7WWn0Bjlp" +
       "hJXBAkXxzISLlUaPkWBJ9vAhD1DNDH39vgc+um77BV5IgCzdCE/BezLsjuGK" +
       "bsjI\n37Z399jQnX/cifdbFO+CR2Ho6Tkk2zOXSvaQJdXNq1qWr2iZPWtWS/" +
       "PS6pWrOOEeTayGMGPCyr5n\nTtnAYBKa9JvTI2JhhOTKdLohBSXZyavQd/H4" +
       "0xeruluzGpNKP/xz6flNsfpG8oh8mhHNqT3aDByA\n6035dndN+8vG6bOuvw" +
       "c/7TaCu0CtUpoFSE2psj+XgfsiT/LLDfoG0DVGIeu+ck0ZiOZKUhtI2nol\n" +
       "m7aOEx8sANCZlwjpcE0ZCihjuTCAcJq3QIiWyBOvvnfH22s2fxlvjw3Cc8fP" +
       "81cRLM605oHojSat\nssIttBwqkZKuJTL5j4Pf+fDa16fj0wKUSmyCcV04JS" +
       "QYQy4Pqi8N/J99T5++4fkSxbtMKY8nw9Fl\nYZzOrZRpHRkVrR/i0Z6Uvpc6" +
       "4KqgQjZYiFG/wWJnzqY0BopdDMUlepKHt8mRD/DUwt0uaFBLMyOg\nxtzjgW" +
       "lUK2IuSX/3eHog32GSjIs8Q5aVnVj4YujVOj3ntCOWnTQLZ1lJmIloDrishM" +
       "7H2p05wekk\n2y89PmBbeP/lNCfiTg2RP5k6J65uVOO6Bm4UO6nDuxl0R/Zf" +
       "/UOjvtoLRvF7wmMFVbW2a4kc/t2g\nd5fOPvxG4Q4WwL/j5GcIxuVEoiWi/b" +
       "jlvZfm/m4q0X82rZF0ttqSZzTekI2huis9Hy5GNrCpAz5c\naWPodPkjZOYe" +
       "WXgJGLHYKf45EEBcIBlSU1Y5eg4Ub8XVZFOqCjuf+jbZk6CzFlTRZ0FQeuE8" +
       "jreT\nQQ2KuHMMADyDLRvuDjJmvN2aUoImRLwOARBWCRnzyQe9r7GpqWNq6p" +
       "n7RpuaXlKjLxpxUhA8a/Ju\npjOHfzYZBoLccZ4U+X1lmYQfHNsArEbKtpWO" +
       "2BZ1yDRETeYJHhDYE5WlfC/ZhKQ8FtGzvd793wvW\nT5w4+gKSOQaHr2RnHq" +
       "ojERTONaBf+ZzIRQp5piZiNzdKBtIxy6gpZOARQmjWaL6QKIaz7qrCuO8q\n" +
       "GBpEVEyQZyfMpkd6Ru9ecMkXg99lU17H8diaM8eJYWPabgnM+v1PJpCTWglk" +
       "8DIruuPIunjjMckJ\nIEsymSXvuUdCWXOslcmkduN5lVteji/ZSFYkYvq1AV" +
       "w5atifTq8u+67XcvwORjmNmPQ7FCykjMZg\n8b2Ziu89hvjeZi7Ov4JFeGca" +
       "AirzLvejUvExYFOiM5G8KkHWSuf99tlfvrik5xFK8AGpHsTngC6U\nMMj9is" +
       "IMIHnQKaGaPoQuj0cenf2rxNd2/DsdYwGxPSnGHH3LUqUpPkSVKVD3JUOXd7" +
       "F0MI3QzcVR\nOuj6Vl3nvNEpeJK3Y8b1Qw2g/W5cftUiAwa29zhgqNVgrURX" +
       "ysZgeW83wCCXKVlQleZweZzHRXkZ\nC+u/cfMdqgdncgNr41U4acC/4cMmOK" +
       "3M+zTcDsK1K+yS0tFCdfZTK9YPjj/1d6ST65QQwac+E1XJ\ngKvEFSp0exNH" +
       "yJF6KFA0Qh60ELKHJBJ4r4Biv5I/5YCZKmxhnVPMqT5vmaqpIo8XS6zIysUi" +
       "S7A1\ncQNcDIJpuqj5fcHXa7DTNV0ypKY0O7JBXTRLSrdDUXPDyvGq7RUGQS" +
       "hedbdqO4LF/SgUf+5jpwBw\nPy7kNKQCqSkhBsG0E2eSps7gFdE/vGL6Byav" +
       "YR5BSlhpHmFWmkeEleYRYaV5hFlpHhFWmkeEleYR\n3VWXkAQya66FHcGheA" +
       "MalcLdx0ZzoGZJAN/ZSB5o8hyp5F3qSPI26YmPrgUPJvoXNncGCF5SLhAc\n" +
       "VRnkpfc1NjV1TE09c99oU9NLanSCj9EJzqTzMPQuwQlaJSEoBhj0PgOK04zW" +
       "mN5j8V1uEYYO3rWM\nkWblLV+OtVQA/kJlxZQB+9lT0pvzx5p6v9P0M0NrGN" +
       "V4hVGNVwTVeEVQjVcY1XhFUA1aA0WTpvgx\nalM4+XbAJkQVQHQKx6mTz583" +
       "JEi4mvcpzSHOicIhRXh2dTtcaUVwoiWS/CNwAHR7WsodS463YMro\nQRR+SE" +
       "1Z7y6JW7dlkQ410gn7cWvC8VjUqR8taWJwhGKNc1QAfD2Wg8uhuKKPnaYNP5" +
       "q2d6Ye4wgT\ngtCU06zYpt04VTQ73qmaVZYk93kEQ8HGr2ds/HrBxq8XbDyT" +
       "LFYiJItZa3pJjW7jr04TG8/MyZb6\nULRAcQ0UaaM5NvJb0qL5MSTxPIWcYx" +
       "Al8XKXWe99EUWYXtSaAY/pvlWk+1aG7lsFum8V6L6VoftW\nge5bBbpvpXS/" +
       "Q6c7OykL4bdiwsMKpORGg/B3QrHdaI8Jf1cuwsNpo2VSwl/mKv+9H2Q3c+Ex" +
       "0feI\nRN/DEH2PQPQ9AtH3METfIxB9j0D0PZTo39GJbk7JQvI9mOR3Q/F1g+" +
       "QPQXGP0RqTfG/a3mHKtw8a\n0XWNhA/EkxpgMRuwvKnz9gobpbpt6qy9kJla" +
       "YgqaPX0oc00K6Sn7cB/D7Zjznr6NyVjUqa82zCUj\nJk2MmDQJYtIkiEkTIy" +
       "ZNgphYajBdfvD/DpmhfTNdCzGG2IEZgUT9OrroYm2JA30AHNFEpnAq4UAR\n" +
       "APj7PQWidYsEfZcYu0KSlWoofvqZleKTTlm8PrLuNMGq9V640oqwSPaRubvb" +
       "afJhMJ+Nh4QHkedK\nhnR6hNz6ZDCuJmKoFlyk0/Wxz8egB0WpcywAPATNfR" +
       "Ai+eBBu+807ukgAuCFEFUZIkfva2xq6pia\neua+0aaml9QQj+urgiogq28Y" +
       "nTn8g2m8y7zjvOX5+nKR5wnHOgCrlrLO2SYht1WTj3fWh4RAZ2Nj\nhNkNhq" +
       "fMC6WTKsxWre8sBROWe6Llm0xJPtMg+VTD9vomYrLjnMH7zTuO7FQV7oPLnu" +
       "wANk8Cpinr\n+rQ3657u5g4gQ/hz9GhXnFaBCD9fTvjZlPAXGoSfZxL+PEz4" +
       "8w3Cny8jPLzwYFF+ws/XF/Mihs62\nJqOWtbx7stMTXwzRL9RVsGhEr5ETfT" +
       "Elep1B9GUm0fFr1nyLDKKTu7x76AxF0m7CfWTB+XDfrDKf\nqvmGGxbRfMaK" +
       "NNM05ZOLarpghClUg/BOGmEmNipFsE8G3jMZmSc6YaI8u7hmA4aYq+Os7zWa" +
       "SM8v\ngm0wkL6QQ7qaQXpxMVUWBlhKHRRqzyBcUwS9NBCuc6BkANdo0EQZYk" +
       "YExirFft1q7g3L1jycln32\nler/Bx36rKsM8SFmioVvN9zSFAso/8NdukTR" +
       "Vp17LJNn0yV8dzkNhw2nyEQiPiYS8QmRiE+IRHxM\nJOITIhFaAwX/AAbepQ" +
       "MJ/E/AxZAIUxwafxfu5O/ScfSWZRQNt6tas/jqA/pCan5kDGCmTA/SGcyD\n" +
       "FeRYvGMePSLy6BGGR48IPHpE4NEjDI8eEXj0SB4egRj/FC4Zjw72m0dBxCPZ" +
       "wSXKJH5oGybxYCeX\nSc+ITHqGYdIzApOeEZj0DMOkZwQmPZOHSXDA6Um4ZE" +
       "w60m8m+RGTlgivl6As4ge2YREPdnJZdFRk\n0VGGRUcFFh0VWHSUYdFRgUVH" +
       "87AIjjf+DC4Zi/7ZbxYNICySnqumjOKHt2EUD3ZyGXVCZNQJhlEn\nBEadEB" +
       "h1gmHUCYFRJ/IwClLrnoFLwqjS0QVhlHmuWGTUcMnwHKNgW3aYBKwAjPL4nT" +
       "KqdIzAKFRl\nsIXe19jU1DE19cx9o1hjxyhI33geLhmjZvebUUMQo1ZZT4HL" +
       "ucXPwYZbPNjJ5dYckVtzGG7NEbg1\nR+DWHIZbcwRuzcnDLTjZ+zu4ZNyq7z" +
       "e3BrP2Lwez+CnYMIsHO7nMahCZ1cAwq0FgVoPArAaGWQ0C\nsxryMAtyqP8E" +
       "l4xZV/abWSOoajnhGD8PG47xYCeXY50ixzoZjnUKHOsUONbJcKxT4FhnHo5B" +
       "hvbf\n4JJx7EZ7jvGPhekmkNk7vBtgpqR3p2f1rupQE8wOQKuKABw/jSvd1j" +
       "8soN1O6KL0Zihu6WPHaSdJ\n336CWtrNXjWaBr9XbVah6B6IZ0rZTlHKdjJS" +
       "tlOQsp2ClO1kpGynIGWWml5SQ3b2S3GSzb3slHIS\nGgo4SFQKYXzpfbgHjK" +
       "K5u1L6fcwBXEt2V3IdRpG8iVuUyH4fZk7ruyil8J6C0u/hWebdQy39msR0\n" +
       "nSGZIAYw04uG2oC5fY/1MN1SFpoe9py9Q5cDk7cENya/KNCaTMbVcMKxCd0m" +
       "Cvc2Rri3CcK9TRDu\nbYxwbxOEextrQjWlFBCYYmLgjtdw+4S5X176ZH6uAd" +
       "i+HNoiTglDHDhFKShxQhA6/QMuRgjT1Mq+\nm+6HEwL3NkPSu9NDkJwTCreB" +
       "Cjj1Qcf7hwS0+xCzEl5nXfpxHztOO/FBpRiztCsXdFx0QcdtXdCH\noox9yM" +
       "jYh4KMfSjI2IeMjH0oyJilppfUEBfkH5nO64JYOkPxCbQbBUUI98C5IP8Z6b" +
       "65IPr6IVEe\nC+aC/BVQjMGzzGuW/PyL5akL4ido44J4sL66oELTw56z/zRd" +
       "kM5bl9byuCjJxxlJPi5I8nFBko8z\nknxckOTjNv5Gn647xsLtmaa/8U915G" +
       "/8Z+dQDXFKpzQFJf4GeQVPCC5G4tK6RfVfmu6Hv9GDHqF3\np2mInL+BTJ1F" +
       "rgIff3P/MIF2V2ABgrR5/4Y+dpx24nTKTfTSbjwPmgrvecwq3vMgbHhpQ1WG" +
       "bNH7\nGpuaOqamnrlvtKnpJTW657k2nc/zWIgNBXw30Q+ncPCnEvzX8p5na7" +
       "pvnkcPfiSSWTjPAznF/uvx\nLPMbKNnThpGSCXKe5zQbMLeeR39qUXB62HO2" +
       "jfE8hLfuDj4aMs5IcjMjyc2CJDcLktzMSHKzIMnN\ndp4nS4XZBWPh9lbG89" +
       "zuzPPcmUM1xCmd0hTkPA+sdc5C0jYMLkbi0tSskq82FfLoKLx74XzJkJqy\n" +
       "oa+n4KshULg4nGg5t+Xc+bNmOfZL+xg8ofi5c3QA/CCWBXhq5T/Ux07TTo6P" +
       "mt5pmATbtCs3tU90\nU/sYN2V5QwBCUBDNg4xoHhRE86AgmgcZ0TwoiOZBwU" +
       "0dpG7qWFrJ8YYAC+2hOAzFf0HxqtEaaOr/\nK76zkUV4G8OFUlkM9/ldIO6E" +
       "Eab4a+FlGP53RMq/w1D+HYHy7wiUf4eh/DsC5d8RKE8/VOU/oVPe\n5mUYCB" +
       "BKCPL9xynpA5DD5f+b0RxIH/DmIj2Y14ukpG/pQy5enwlvfGoEyB4oFciOqg" +
       "wi0/sam5o6\npqaeuW+0qeklNYTsgRE62c2PnzBER2BQwivDAgGD6KdDUWY0" +
       "xkQfhe/ye8TfWMZIs+ru5HyXoIC/\nNl9yoaugvcT8mnl7hi409pjqTDJRde" +
       "lf94mqtI9RpX2CKu0TVGkfo0r7BFWiNVA0MW+SYC2RAy1i\n31jBapIDWdBT" +
       "SzlpOPkycFjE3h3CbnA8pcWALAcMYzeUrHs8c+FKK8IyK0DeMeHu8KElXhbs" +
       "69nE\nvQlDOs115t/QEcvWqmFS6XB5FVjL4AfFpc7RAHC81A7AUjuwoY+dph" +
       "2F/CZqaTdrKTQVfi1lVgmv\n4giIMX+AifkDQswfEGL+ABPzB4SYPyDE/AEa" +
       "8wdozG/zKg4LqaGAgD8AAX8gazTH3gXHk/ktS4AP\npPV3BgrSaD+FNubUuz" +
       "kL5zt6BisYeq9l6L1WoPdagd5rGXqvFei9llV49nQzSzTHpILbbblYIh/h\n" +
       "lCUIZwFhJ2UsYv4X4GIEIE2V+WG461/6tZYkX/bkZE9PvhGGxgBC8o0AdlKT" +
       "bwJ7RSbtZZi0V2DS\nXoFJexkm7RWYtDcPk+AjBIvhkjHpF/ZMkr+cJEDYjn" +
       "sOacrg5UsWJ7uAfK1xlbx5f7zw5n0ehvds\n8HGIWZJZOv2AcJLxamq6OxzP" +
       "OvZoB9yTAmAPM3r9HNbrX0PxfB87TjvK4CGopV15tAOiRztgejRP\n0pTT50" +
       "Q5fY6R0+cEOX1OkNPnGDl9TpBTS00vqdG9GX6+fK8xn5wkhuIFKGDNGTiKm2" +
       "PklpJWUL6P\nmw7XlBHGhx1YGcTgtvvauK35DQRIjZ9JBR46/yUUz+rfQAh8" +
       "ggGdfwMh8IH8GwiwujtHqgMF2zkP\nwOO1wN8wgfJ7sVckRnekZIIYwPrMVg" +
       "bWx2e2BaeHvVC9BPJnypS7x42GVjHqc4BRnwOC+hwQ1If5\nyFlA+MiZUQMF" +
       "Wpx4k1PMubpjKSqCfjNCDAYd7ZkHB+TQR24+pzTtJC5yChKyZXAxgpbWzXfw" +
       "grSt\n+c79XmL9FRlCz07Tgqxvi3Hn2ILz+44AtLkQmgdh1RQEfgSruRfFIA" +
       "CeZ6jK4BC9r7GpqWNq6pn7\nRpuaXlJD3EVwFVTBydXgMjpz+Ac/v9xl3om+" +
       "xGANvA1mmZQ1RVpx6DRkHG5QXL0HmdV7UFi9B4XV\ne5BZvQeF1bu1ppfU6B" +
       "RsNyi4jqUSVKzhnGkQXmMV+CC3z0xTT8gcMwagxVIaF+aYcbBLocIQXMa4\n" +
       "ueCVGA089WKsqZBq8Wsqs8o8sB28yBBViSQSbkAJaQ7BZl0+iBUNXoZ5xDPi" +
       "Wp4R8vPelBGYRAWn\ntuRQd3BLXtcT1CSriSqJcEhWEzIwZ69/EFcToiz2fz" +
       "XBiDx8rHx0MUX+LgsTcpprTg6di507522I\nPmPG5jNmbL5gxuYLZmw+Y8bm" +
       "C2Zsfh7nPQYRuwEuhuBp6vv6f04/2BHOdixORmUngIZIhuYkj2aA\n8GAn9d" +
       "xqUDyoH2QO6geFg/pB4aB+kDmoHxQO6gfzHdQfj9BfA5eMSf0/qD9AS9YmE+" +
       "05d4v44TlG\n0d0iHuyk7hYFxcP6QeawflA4rB8UDusHmcP6QeGwfjDfYX3I" +
       "WmyGS8ao/h/WH6gl8dm6nJzix7fh\nFA92cjklntkPMmf2g8KZ/aBwZj/InN" +
       "kPCmf2g/nO7IPda4FLxqkcZ/bl+3qQZz1J0iPxGxawaRIw\npzENl+uzyebp" +
       "E3yBUYxpTrjHDxVlJeaaqww/hSyDp5BlgT52nLbfrONeC08fn7tYW54Q15Yn" +
       "ZNk8\nsBoguZrKHvIrxgApmCjdvQRxsvQvwx/YsKbh5CISFEEo4GvnZWOMHu" +
       "DhURl+P52rd1vD9jEcti6D\nS1Mqlsfjans4Xq3Lx9KeiJqCr7KRreQZ2Vg7" +
       "2UqGj56SveQcDfjVIySmzZYIq9u9KLo3Jwp9Afai\nHHISA+D3CpcNlZjNEX" +
       "aqK66Bi4EFkztcNh/PH9e6yh0erC+oCj0/JjQsW+DJr4jEvItv+ggyb/oI\n" +
       "Cm/6CApv+ggyb/oICm/6CDJv+iibwqRZsDrmQLMAeBKmu/mBNyohcDsbV8zB" +
       "jDlFkZf4NuRmPK1w\nMXKQ1k10Ga5049sqiNEReuR8WwUxGQKYpqzuU+5gl9" +
       "2HnKTerSziHkOAbWcMdwwzG/Y/yjr72HHa\nsXczP5fk2L2hqfDuzayy5Ce6" +
       "9m8x6t+uSytCsmMuOkEBXzEuA2tRdo3RBXZwN7hTQ87/wFuG50oE\nyq3/gb" +
       "OlM6WC+T/ifzba+B+peon+pxhYsP7n7nT//E+h58f6n687NcGGWpgmGFUZBp" +
       "fe19jU1DE1\n9cx9o1gD9zvYzD1WB1w5oB2CiMDtblxxF+bMKYq9xAHBLp4K" +
       "FyMIaWpF8bdN3TggeE3hREmPnAMC\nsCkSMKdPJYQMaufOZ7977AD2IGNUD2" +
       "FGQxpz2S/72HHasfPRE4hduJ79ouvZb7oeI0fbteM5RB3P\n62mFS/fORSEo" +
       "noUC9lTK/mB0gN3On90pH+d24O0j50nEyK3bOUtfD4ni+D/idl6ycTtSpRLd" +
       "TjGw\nYN3O39P9czuFnh/rdj51bHj3i4Z3P2N49wuGd79gePczhne/YHj3M4" +
       "b3mJkCzmqAK6dzTBAQuMXH\nycs+wnw5RXHnnA5sal6ARQRdjBikdetZPgmD" +
       "uksoz31uT//emzCk8/wE9qUl0aibVPLyyQxmUJzp\nHAEAPxual58DxQwoZm" +
       "qKN95q8BL9zvMSVRmco/c1NjV1TE09c99oU9NLaogjKK+GKni4Xj6bThz+\n" +
       "OR/ud5l3nDmrQVeThBsc0wBsnZRpzoLUKLzc3Bnb4q1M9gJQ2BfVTQzzyBO+" +
       "db9aOp85fXrkibs3\nn3iWX6JgkprsxbReRmndYNB6uaH95UswvZdyFrp8Zd" +
       "qFhWZQrENXfRFRXKNQqSmfzVjt8lUYCzxz\nTlbWoyuaX1YA7ErpxJucyYoa" +
       "18J9FZZS3JojZRhdkSKSMiKVFnpqorzDkJawKS34u2bl8F0zvPjS\n7/I9yA" +
       "roxEi7WYQiq8cvQs0qnGEFvrB8jmFFdrFCbVq/ZcVSd+i8xmAgamt+3aD8ks" +
       "LrNEW4gVfV\nq3lVzf1pA6xFhdZHMj020aX8GlZqTHZcUTSNgt436Pzw48YM" +
       "QyKFVxvKkA4HOgBwcc4wwSpmlp1h\nMiOUMTZgjtJ6GHMCO7aVRTQne1i65F" +
       "x8WFX3M66pn1mp7+HtkPASUBefWBldRMH8oa2JelTJn8DA\nTHSBgr+eWLSJ" +
       "PsnRWHqQdj4avBsuZgJpGh38vOBxzxcU8qICfkin7wzm4546u+c80sDnaQY1" +
       "KPa7\nC3zwHlI57CGVwx5S+bNc4HNIDHwOMYHPISHwOSQEPoeYwOeQEPgcEg" +
       "IfugNWDjE8WV69wAY++L3y\nu8w7zmcsR9caCTs4rgHYZVKuOTRCgn2155t1" +
       "MfusPPJZpQds4oQKozrHsOr8gVvLHqXEftsg9n+a\nq5LXMcGP8GbsXd6MOY" +
       "x8VqCroYgovq9QsSl/gY18jmMs8Mw5YbkcXWp+YQGwTunE1/TVY7mQFmno\n" +
       "06rHbEWiZcgjFZdPdXEJ+Q1xOWGKy39j6cA7l/ebd/lCnyClRtpV7PO0GPs8" +
       "LcY+LxqGZBcr1qYB\nPFo0jYfe3zBYaF1SHSu8WlOM3+a0NSS89jV/8PN+oV" +
       "VSXFmEKli5MfnxafGUCrr/p84Qfh0Y8hRe\nc3SOEF3JH/2Eym2iH6lxEqMf" +
       "Hqyv0U+xLMo8li45VyBW5f2s6+pnV+75+Ce0wp0pEuKfYonmSlsr\nVe8u/o" +
       "Gvr3++iBO9jKOxNP6ZhwbfBBczgbQeJIQ2FDz+QfgqSyVDakpjn+KfJfJEA1" +
       "n0EwoziEHR\n6ir6CXVgqYQTiSHI3gp1WqMf9Dsf/YQ6zOiH3tfY1NQxNfXM" +
       "faNNTS+p0VdYeM8SVlihFBP9hOBI\nOY5+9DvOZ6BAFD8/4ZnB8QzAmqU8cx" +
       "azivbVjmuW1SwQWBb7NJL4RzKdwqjNdgVT1LqYDV1PSX2z\nQeobjUVJaAsm" +
       "93W8CbuVN2EOY59aEv8UDcVdChWaUIqJfUK3YSzwzDlRQdEvfniSR1QALCad" +
       "uMON\nSNFdOZYVaeSzgUQ/RaPkvVJhuZsKywOGsNxjCsvXsGzsSeuRj36XL/" +
       "LxE1qk3cQ9yOTxcY9ZZfre\ntGFCdrEibZq+64uk69D3DQb7LCup0PbCKzTF" +
       "92ZeT/+d19O8UU9oV6GVUbKeeJyVGZMbdxdLnaDz\nXp0dwtrv3sLrDOXHAw" +
       "4UAOAetIl5pEZJjHl4sL7GPMWyJb9h6ZJz3WFV3M+2nn5WZV6Id/7qzggJ\n" +
       "8U6xxPJdW/v0lrt4Zz6JeYo20U84GitNcA4SUhNT2YwyCqSnPZ7UZmibUmp2" +
       "RkM40hluV1siE17Y\nMPXp1JBnvIqvFi041QicyFCCtUqwrTseR/PCmZF+9B" +
       "tz70czbMNHOZQgLgfiFfcARVOCNCsSDQ5/\nYIKhE+TnEjQ59DPc+oj5Ga4p" +
       "g+GEIXlTWH3rlWpE0y2hsjGjjIXOyIRTmVgiEkuF4zMa6N2ey/4l\nlZhVHf" +
       "MiYqdE8meUkWbzi2Ntdd0NiyFvsyXy19QVF/1+5R8egoYZZaJBGrVHm4EzPf" +
       "VGZotLv3tZ\nVc/O1bd7FW+tUhqJhzdvhvECtUqAaCAe3oN6G2/bG+0r9sff" +
       "tn1l69FBXmLXlYGGEhD5J6hX2vcD\n92sGLnz5kscff9DEvcJyDBy6YNDHbZ" +
       "Ynslo4EVFnT/0g8PGz733RSjczzVu5qef/AquDMLkP4gAA\n");
    
    public Date(final jif.lang.Label jif$L) {
        super();
        this.jif$jif_util_Date_L = jif$L;
    }
    
    public void jif$invokeDefConstructor() { this.jif$util$Date$(); }
    
    public static boolean jif$Instanceof(final jif.lang.Label jif$L,
                                         final Object o) {
        if (o instanceof Date) {
            Date c = (Date) o;
            return jif.lang.LabelUtil.relabelsTo(c.jif$jif_util_Date_L, jif$L);
        }
        return false;
    }
    
    public static Date jif$cast$jif_util_Date(final jif.lang.Label jif$L,
                                              final Object o) {
        if (jif$Instanceof(jif$L, o)) return (Date) o;
        throw new ClassCastException();
    }
    
    final private jif.lang.Label jif$jif_util_Date_L;
    
    private void jif$init() {  }
    
    private jif.lang.Label jif$jif_lang_JifObject_L;
    
    final public jif.lang.Label jif$getjif_lang_JifObject_L() {
        if (this.jif$jif_lang_JifObject_L == null)
            this.jif$jif_lang_JifObject_L = this.jif$jif_util_Date_L;
        return this.jif$jif_lang_JifObject_L;
    }
    
    private jif.lang.Label jif$jif_lang_IDComparable_L;
    
    final public jif.lang.Label jif$getjif_lang_IDComparable_L() {
        if (this.jif$jif_lang_IDComparable_L == null)
            this.jif$jif_lang_IDComparable_L = this.jif$jif_lang_JifObject_L;
        return this.jif$jif_lang_IDComparable_L;
    }
    
    private jif.lang.Label jif$jif_lang_Hashable_L;
    
    final public jif.lang.Label jif$getjif_lang_Hashable_L() {
        if (this.jif$jif_lang_Hashable_L == null)
            this.jif$jif_lang_Hashable_L = this.jif$jif_lang_JifObject_L;
        return this.jif$jif_lang_Hashable_L;
    }
    
    private jif.lang.Label jif$jif_lang_ToStringable_L;
    
    final public jif.lang.Label jif$getjif_lang_ToStringable_L() {
        if (this.jif$jif_lang_ToStringable_L == null)
            this.jif$jif_lang_ToStringable_L = this.jif$jif_lang_JifObject_L;
        return this.jif$jif_lang_ToStringable_L;
    }
    
    final public static String jlc$CompilerVersion$jl = "2.0.0";
    final public static long jlc$SourceLastModified$jl = 1149825579000L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAAKy8a+w17XoX9LzvPrVr77b70NLS0iMbaB26Z9ZaM7NmtQSd" +
       "86w5n9eaKc3rrDmf\nz2sOECPRUJSgGFuDBAENxkT6QdTABwVNQI2KHyRRwg" +
       "dRhKBRMUFUaiLirP97ft7n3XvD7pNc85//\nzH1f9+++7uv4zzPXr/7tV5/r" +
       "u1c/0dTFEhf18I1hacL+G6rX9WFAFl7fm9uDd/z9vwH9hj/9u/7M\nVz7z6v" +
       "vcV9+XVsbgDalP1tUQzoP76ktlWN7DrseDIAzcV1+pwjAwwi71inTdBtaV++" +
       "qrfRpX3jB2\nYa+HfV08ngO/2o9N2L2s+f5D8dWX/Lrqh270h7rrh1dfFjPv" +
       "4YHjkBagmPbDz4uvPh+lYRH07at/\n6tVb4qvPRYUXbwN/g/j+LsAXjiDzfL" +
       "4N36UbzC7y/PD9KZ/N0yoYXv346zM+2PHXhW3ANvULZTgk\n9QdLfbbytgev" +
       "vvoupMKrYtAYurSKt6Gfq8dtleHVD38q023QdzWen3tx+M7w6odeH6e++2ob" +
       "9d0v\nYnlOGV79wOvDXjjN3asffu3MPnJayue/9Pf/efXv/cTbL5iD0C+e+D" +
       "+3Tfqx1ybpYRR2YeWH7078\ntfEbv3xxxt/09qtX2+AfeG3wu2Pw3/JnLfF/" +
       "/o9+/N0xP/KGMco9C/3hHf//RX/Tj/5l/G9+92ee\nML6rqfv0qQof2/nLqa" +
       "rvvfn5udl08Td8wPH58hvvv/yP9f/U+af/7fB/ffvV5y+vPu/XxVhWl1ff\n" +
       "HVYB+d79F7Z7Ma3Cy6vPFtuPbedRWoTPnX92u2+8IXm5n5tXr159z0Y/+N7P" +
       "7xlefRflDeE3sjQa\nXv3GIr3/bN/54Pbbu/Defzc/p3/P9NZbG8jf9LrBFJ" +
       "t2cXURhN07/r/1N/6L30ML/9zvf/sDlXlv\n4eHV92x83t3Xk+urt9564faD" +
       "H9/yU4bBU9X/t3/357/8L/xs/2fefvUZ99V3p2U5Dt692Hb2Ja8o\n6ikM3h" +
       "ledOQrH9HHFzXYdOhL902dNs18p9gYvajvtvFH9+qnXlebD43tst15my78bl" +
       "R+9Ss/Rv+R\n5wk/T+T7n9zfhbbJN38X25d+xvhF/p/8/T/1meeg6bObHJ87" +
       "+fq35v6Or37tB+R/8//+kT/1rga9\nDkjtaj8MNj/x4YR3oONPyn/i8P+8/e" +
       "pzm3Fs7mHwtgPebO3HXjeOj+nzz7+n/MOrn/qErb2+yM+/\n70eeonpbfPXF" +
       "qO5Kr3iyed/4d0PS1dOHT1704Ysv99/7D9799/+9R/9g8xFkXTab/nU/wYYb" +
       "1u20\ng+ZdDXpefuIp1tc2/uKu/u7ll7i/9V/+9C++/VHP9n0fcYFGOLxrJ1" +
       "/58FTMLgy35//dH1b/5V/5\n27/0Cy9H8t6ZDK8+34z3IvXnF6A/8NamAl97" +
       "g81+44e+/5f/lZ/5o3/l/TP/2ofc8a7zlueRz7/3\nL//ov/qfef/aZs+bjf" +
       "XpGr7Y0lvvnf6T/1eHV9/7VPGnKn5D9O5h8f66zyvwcv/bt9GvXua8mt97\n" +
       "+1TM102Kefr494+nvP/u/+sv/LHdT7wL7znnh184fKX/pE/72MR3/PXPW3/s" +
       "1/6r4a+9SPRD5Xny\n+LH5k8va3kc0FftvH1/5/L/zx8u3X33BffXll7jkVY" +
       "PtFeNT3u4WWXryvYfiq+/52PuPR4l3XeKH\nCvmbXlfIjyz7uip+6EG2++fo" +
       "5/0XPqp9myC+utGPbfS8/97nwy8/L1+Z33rVPG9+7mXgb365/tYP\nVOMLTZ" +
       "c+vGcIf/XZJdxi/ieFqXZpufnfx3sB4l/6sT/5t/69v6F//9sfiaK/+ZPG9Z" +
       "E570bSF7S7\nZt5W+MlvtsLL6L8I/OSv/lP6X7u/6x+++nHnSFdj+T8tfyH8" +
       "bb/jD/71NzjYz2yx/sXKXraKfkw+\nP77R9z3pk/Jh3yCf5/3v3ETzuXLTmZ" +
       "fYQXwa5x/d6MtP+iRn8Ztw/kzgLZ/K94c22r0Xpr76Sb7a\nJ/l+ZsMapZX3" +
       "Eu1/erP8/iVL2xbavWxB3gTV95/MHF4M/N3z/V3/+w96f67+g19++0XZ7l7/" +
       "rrK9\nnnJ9MqP6WKL0ctrf/bGd/Pb3JPXRnbzvMb784mteXMa72VTTNN9MIk" +
       "9p/8ZPSuQX3yCR5731Io7n\nxX6KYpN5r0TXMMxfRt4+AfM3vsb+ef2FbwHp" +
       "p57m90lIwbcF6fv6pO4G6lvh+rHX1vhWuD6/0U9v\n9PVP4sr/IXBJH6rOm3" +
       "F9/bU1vh1cP7vRz3wSV/dt4fre0puf0rpUL9jeAOtn3qfXYBGfAut55D+3\n" +
       "GSb4pI/CeuuDQPuinpdN7eOw++r/+Cf+5N/7vb+Evf0MhJ97PN395qg+osby" +
       "+CyHft+v/sqPfvGX\n/4c/8BK0Xr39c3/6yXT5Zlv8ne9v8SsU7hjvXOR3YA" +
       "h6x6Fx3XjdTzwvP7kB3Gz++A3oG9Dz99/7\nbYnvB7PC//r72Ym9VW1bdv31" +
       "LWi/nNsbVthE8L0fbk2st2rnD/zNP/SX/sXf/N9v++ff3/9z9O/Z\nxPfFP/" +
       "N3/t6vPX/5fc/LP7PVQ8/ljHrs/FD0+k2dgnTzJcFzxZd1/onmXUdAbkGo2J" +
       "i/aY8/939y\ncH/B3/+n2OfwOFn7+xE+sd46YiX5UOyEVOjigre1iuveAcQX" +
       "j74YFxbXKPcRGHe+7DGMJ4ra3uka\nK3bwkKbw0UZP18G+Nt3Ji4Z90EY2ix" +
       "CteGu78w1oO8dGm9sd6Cn0Id6K4HSqVj5HfAx1j26RF9hD\n3R3n8ypl1Ul1" +
       "wAdwXNVIAqVjF1pQObPsbSg69sAOaVm4ZeBz7NCRbLh4nZawAQCxxXWvMyfn" +
       "XO9B\n0MHE3TFbY9tq2uLOT8PZ01fJgEy3Xkb1LmDSfVpZeGCas8tiQ0seRn" +
       "u6M7Qd99XAJI6FqqLU56Op\nF1EJ8cFu2NLgQ77sdWR0yTIib6Y6O5R0Z/qI" +
       "Ohhzi/Th/nDXlywUm8VI7OFiAYJ5ySh6KQPldO4h\nBAzUg5o0Wr2Lz04THw" +
       "TethsnhawrP0seWwqB1BLi6drY0rlqPcaQu0JheUbbj3knz5I4H1iHLuml\n" +
       "t2C3dIAD38tesLPuUre/TssjofHk4J2sQ81mUN55iEXkhklubAPIvbhSLkHH" +
       "k+ewNWiAUHeHR+7Q\nCZ5/L3kBY3r2rKXnHZak3lVwBo5hGnp/rkSCswuOwc" +
       "hJ5QzXj3FZgyDBSsNkoACdT2yBlnjXUJNZ\nHaR90JR2mnIOe+W7h74T+HqZ" +
       "O9ckCF1KDCWAb95et2UWu+EyQvq07Tg8bkGQq2lqUg3kjego36kS\n87QHJw" +
       "oAoAehF3qkKl7D7uCMAnG2sRbT9tr79sS0dMuqqOw0VEexxMb7g2yX+BLiGm" +
       "eyZMuDANxG\ndxlcr8rET0nsFY+HLcgeedB3JhBcM+6QAGXiktaUlXRTOhiB" +
       "kYDrhMQDus33Q8Pjh4o9QSQFKjWF\nAUN0ua12H1oPMLWjVWRuWM6G4Lgz0v" +
       "m2DDodiAzAQdRxuLP0WKzzHNBRwhV3UYBJpEKgamo6LkoB\nHrXqJSegLDBB" +
       "ajAsmBByiVI0tuyEHcql/dmdez67mKAVlvoR7OLQo7oHD0Ok1oXElb8AZWv4" +
       "Xua1\nToGNUJacUOzMC/egiuU8oLl9YxC1Ujs7EUhs7JZM90UZBFySa5TSI/" +
       "BITD17rBr1TNviObAatxPG\n7jqNxzlhShwT51C9tHLQpo6eu1MfZ/swJ3ad" +
       "lfo5axxmhuF0UTaX3pOkmr6xe+VclGpG2ywocyHT\nd8OCBkfcCamLr6SBiR" +
       "/1AVGlu09CthtED5bjdglxwce+5VAmJTUycEy+zE/I/dJUUMZkK7AnAzXk\n" +
       "b0bgJ7HrhvU95MZR9qaQYRcFZvuJI7ftXwWKNtKdPCztXI42LdKU45miTfRU" +
       "qyXjJuXMg5m7ToEp\nTmHmAPE5yd4ks6VchTTWHBTMmrwZErvSaXkGQulO7X" +
       "RpRtrMzvOE3qf9cjt4RLF/VJl9no/H1L+I\nqEl4ouOVae8kdBPGy14Y9tTN" +
       "og/uhYjpqCGZvRwTfIbSuxvTL9B8VbQ1tPjR9vTFRQq7pJKbEz8S\nQAoJnd" +
       "zPBKSHJLm0jEaRSkr5Vg5TISeKXhbtR+ARevK0lx678+lSBjOVxcWm/QUXXz" +
       "wP7iS0FtoG\nPF97OQBmwAOH4sp4l56LIYS7oldAmQ0cCI90rOb9PqFLaXHO" +
       "5rRTAMDbP3qD8d3EHUG7tGaY3qx/\nPVqPTN6vF25SpUy/AHdlXo3m6sdXtZ" +
       "FxouYX4nTwF3W5MLLkH5u7j+5K6Ippx3vlL/LxoZ78AwMM\n/KiuXcDSa4bf" +
       "nYChTplwnSLxSldFC8j3QD5CKEoKDzi/9pop6sljRsWL4O8C2Ip7Z5RvENNP" +
       "1+ve\nAsgZfxALdogmwlARJcwyat/PsKyAFcildDYDgtCUF6RwyI4yOo+PjD" +
       "0qLXgZ7ugLoJFGuPhXw0nV\ntp8zbDxZRcEklzkuLwvS87RK4oy8pw/lSaeR" +
       "ckYw8MhFDyUTLIOai+JOylTGzdVpd2qNsw0tZSSc\ncstum6Rh9ryZZmvqB4" +
       "B/mllYjaYmtfkWJMvMU2pbEsbQvoiEzZ7qWzhZW0g+9oQudPquoR3+ISun\n" +
       "2zGJgjVGzo10WS8ucbx2SUYyZUejZcZsR5nZIPDIo/tjbVtJ6uFioohH59Oe" +
       "zNr3Q7WZHrOTknjJ\nr8oF8klWuCalu6+bVUJ5PjLFPbxH7lHnDjOU94DkCt" +
       "fHfJ/O4FmitpBzbuW9HacmXINCqUvXfthd\nW8FbxS7piAMYgQCH7P1plS7+" +
       "5qTDRKN5FSV8V8DXq+UddauQjaMz76nHAxYWfa+BV9Wq3SHQaxU7\nCDs2CI" +
       "05BH08isCThKlR6AOHPlgyxj/HBykTx60UUjUww/pH06fq2civi9pEVXSK8k" +
       "QrikcSNb6F\n3mBh3LHMtKDzwjptZ1WN5ahNPBNFJu7JTYcpKFKqM7d3lwPs" +
       "xqLrJqrHXtX9dM8gtZauAwUuIKnh\n8MJPtpXsSj327tqqAmMjjQfkfC/rEI" +
       "PVU5SdUtEAYaHEcqL2pNvcVzU6H6U5EBdmPjRnPGWpAY6c\nLsxMeqqY6bbz" +
       "MIHdL47X+/hy0krDuz4EdkxzASn4wNdS+5aFtl8ehJFXWkxwzyO8uTPG3nyv" +
       "7eMu\n4LCC1rblIJHWfndN7NG8Ila7r4/27dAEJWvvE068ZUeZZgbqRHgcDZ" +
       "c5X6QnEG1AAK1kIbJi5p6L\n0s3fN7qmCpHUMkuQ7Uj3MLrHI5LUJ66Q1K6X" +
       "bPzWWl1hLTXQ8TWZh6Rjen4+J7Mz6Ye6DkD/PD6O\njw7GvCOrU6yQxAPrAR" +
       "i8u459liFCsKl9beRudjcZp06F6nG49d0YPACwvfIlX639KZrbKXpYmes/\n" +
       "UiFb6jlu4EvC3Uull+gLNsq7PFrbgfT8k0Vlj3kGveA42hjQ8oq0prN4PmPh" +
       "7Shmp64BsZBy3QmH\ncT+qSAZjG2GPdmeDg27I6rO0k+zuNN/FTOIikLnXg1" +
       "o1N92VCFkNSFKp27yv85TELEc+0QBW4gi3\nn9Km5OFb7amtVrecxXNMJ3SM" +
       "H5fiDjQa0eRp+0ZCYQncmiA49yAQR8B8QmOx8yDDG4uqPOg4TB9q\nSeSa7B" +
       "TF+YH1Fy9IDIpdVqjUiVuGQt3udMer+nZVWVqa02EyxpA8snAEP47nR7H3tx" +
       "OICXQG0qlp\npgcx5N58ENC7Ip6thg79x2P/SH1CkMTSzOCdUcL2qtcBB63M" +
       "cKHKqcqtsb3rMc8luSfksOKUFLft\noY0aWKcMpHOdS71y1QW9jvsSM2ABRt" +
       "tKThsa3mH5HAgyFbR0lIJ7EZmIUeXsHDlKQ29Dk4VPRplZ\nyO0uJqbDVxwy" +
       "14QGAJFanR6yMUTeYrXSoHmJgci7gIvOCOpVe6OMRsBW8zw9yDRCtTmTa8Y5" +
       "INEz\n6zYMGh9GMAwO/smMMPzUR/fJPJX6qB8PVZf1wY2l9tludYiUJEWLl6" +
       "TFm/etbXqFXooXlVVkTfVS\nL744R610kCohFDhVuwMjMK53GSjHJf1AFvQB" +
       "OHs8sum9vYtcjZZv9MFQTGQ63bDZPg49AJ6Pj0Bo\n7aNP5VdshlQa1sdUYY" +
       "ljpI0s7/SXjMNP3ZjRzOY9W/hykgWL3okROFN7yjcxXLbche6KfHGnGVTS\n" +
       "OcZQ/ep60IrsQ24vHqki8leFYFkUR4/zccv1N7O5K65i0Lx/oF1/dx9dSRr0" +
       "4B5pq21H3NQkkseN\n59Z3Iz66cIiL+lB8sx82jhlEBdJLNiYhn+RYFU3F/s" +
       "yCqvFQFFhJzsHOAOWV8fGxSMUQuSydgQlb\nsTWb3MksFbVGymN5QsKsgm63" +
       "pPVVnV8KrlotGGhMHjKCyV6xvUJ3I0L08o5rRYATh1sN9Khy9O77\ny/EeK9" +
       "fsXheJeicn0DUhZT9TsDH2MYava5Eq+4Ib4Fu6F5NGDJdLe0z6aYCMeFeR6h" +
       "haFKOSiddF\ntNaaF2pmXHElVWtPbWWGP29FW0zNzmg4eEmLU37T5k66GlmY" +
       "kPyxZ82CtJfKOHb5zuZxVlsXp8k0\nS26beAjP8mX1pjbeBwrcmoJDeQ69h5" +
       "NDR+qwUz7YoJ2K0TmW2L18kIdVdLc61AGpA03s7hfHKjCH\nDJdF6dvlIeX5" +
       "afO1FdNcoQpo83MUS1ADT0b0iEpRBxukPh1k3cf6iQendgD1BZl8Tlmt87jD" +
       "mb6e\nPQCoBZS+nO6NTs/GCt1vFSsYh0RJGwJhjRBCVoPcEyh4UEPbSh/WSQ" +
       "Ydf1N7Nhu9Ur2DPtEv+k69\ngXeYixGmPtwq+9E6qFCoveugpUI2JlxelSbb" +
       "MhSAL7LtnVec6/VKaqFagnF8n3Ns9G4iKpyu4vE4\n7c5UJLTYIs9ajWErnx" +
       "yvSLNPxFl9ZNwpkFkqqoM0vV0FL9geqyK9pEscRqKXbprZnDrvoapld5qx\n" +
       "u+rtvDWmJVr2PepAdMokI/HRvgd641OcBC23TuU4/QTfTtdVK3Kpo+X+QuJw" +
       "BfJbSmNbD65etMDf\n1+SjLPMdISzgtbw4J4gW4At95LNlFUXpnqAI4V+K+n" +
       "ZIOfkxwQx0GAepn7pUFCLBPwjqlmJbLtJq\nZtHf245blnT3oKZoSW396Bs8" +
       "qcrBIaWkeLYcWHMU55nmmifIIrhkb4gkr+uJa9bWgF7copRZbVki\nxCW5Uw" +
       "9eU5O0dvoRDfGbV1JbZjqua2faA32ah8EbW+lY0ow07Ec673uHUYeJrWFYlJ" +
       "ezVqj6I+6p\naJXycp077nxy1YzfIeSBwqItXV+vPq8ParTZVynuU8PqiM0X" +
       "53VkuEGiLFKpG1HtbiU8zlN0vm4k\nUnQlshrPulceSGLC2RW2wppSZrtbVT" +
       "A3ZwEOb/2FI72qH3oGZ6HoZNQkM/qCJtTR8Xi/FrdhYFga\nstejXHd4PBXQ" +
       "EeO0yAnw3Xk+IIxFo6RN6w1LHNISV09KbiWMJoh403Albz/RzEpTpLLV0UJ/" +
       "HBCq\nvOiObeURXTZkt6mjdu98fWeNaWUlppHMEI96RK+vQib6edW6N5+lPG" +
       "OpJX7mAOkSIRn+oKNlOTRQ\nXl7ytGZ6RLbCA8k3dEvoViXvNHQW2JUkrrjv" +
       "OAmkpZwVD6i5rezZNv3ByjmY42l7QwMFkWkjOgwk\nVA7LKYRLyZAdQ2lUPs" +
       "F3oOw7cIeVtArsj/ylrZwt6NuXSYHKPYG4lzbRnULWsFloIVxXLmcdoTrV\n" +
       "p53ucLvuxZW+DEKbyqnpZ/augXRj0299z6mabB00WwC53DPCfXIbcWdEVYau" +
       "0ltVS5jeHAncPxmR\nF6ssdEY8dxCiY2ZukWCEKiG8mzv6QBvGMsc5cMRDuV" +
       "GX4FqZDMUdJo6ih/hUttGFNV2ZuDSWRBgN\nTSbGxQsuW9AtQyMmHVYJrFL3" +
       "uwwhdmyz4bpWEJEVTcLfCDy/zevFEAvLU/MrwZ8feLJFL5nituQj\n9oGxPJ" +
       "0IG7nE5BqQNEiEq9hnIyvc++yyO57c4/mcadUN9NMouDMKQOoHrkbNPD+sys" +
       "14eBof3OR2\n3167Q+qq1+FqbBEJba6dotCh6S2sxKrk0iHVzhwq3CEE78w/" +
       "zMuBTxLFFAKxTgjj5i7eQwtADobm\nsBuLcWUbWVDBeUWjQu6jhsXqBCsTTm" +
       "7No0cRC75DoSMSVIEMYA92S5XGx+SDJ9MjR4gcrKZmDPrQ\n9Blwi6M+NUFR" +
       "3PwbC1E2ewlHwM8hN1VcwUqGy6Gqi1158hR6xnpcJt1iyhqfhlwyTbLRJHGf" +
       "b4/a\nJb0uhPJ4VBF3OPSHaC4e2XQkfSxSsb2vixIKITVXZ4Y37Pr+MgzE1O" +
       "bulumbIpJtJSEQUd4ejTJo\n2irKciF9G4J5Q3O0BVbbFJNB/nw2EQCAo9yY" +
       "74MkOFg1nZli57ir0rl6w+m53elgCrmo9Wh8FBba\nWMtcJ2VYcLpu2XbrQE" +
       "TdBJRkFWEjcANW6hRIWVv0HT2GEKVFiXfzsGbxtbCEw2U0buAx5Or4eGCN\n" +
       "7NJjGuLZlZ3BzpjZSnasBlLZlFAT3X3Bl5sXprq0T6QTt1krMoV5vjss2Fkt" +
       "iXuogoVMRqkZH3QV\nyVEeRUZbUVCihC0FrahxLiAddCPQUrLHUIv6gzlNpc" +
       "TY9+vBg3MfHA7xLkAiRX1s1bMGXycRzBDW\nv8yjTdfpNQXvWfFwoxIs4Ra6" +
       "YYfOZaBhaA13NcP8ckjGtD9zC8LKHdngK+LvvKtaB2qeuYoTXm8x\nz4qcI0" +
       "QwQtNbPCEKw9o7SBjD1h272S6jtGQ2RkB2d3OUXJnC1F2I0cKRxrIuNHeath" +
       "VsnCTh1EJt\n2Ecq48uHfPCs+lLq4qTgOnEl9WpdtPDcndimvcot2lQnvdnC" +
       "b7H5vrmZrQTHPJAfd02mcNHquEQt\n+r4LW8BtX5SOFnFyj3f0XRiaq3Ig3P" +
       "1C6bQZOltCTzk0ci0iEnQTQcOYgV0fwel8nB7X3ZbJwkAs\nGIrmcd1Br7i2" +
       "zmjdVGSPiAB3vehagzSpm8prIBDDZV7QuHE3vFqZXT0R1YSLt57ajhwVrN2x" +
       "REPb\n9KK5SyQ1eYDq4zn0pfwWn8+FtT9xWJB6EhbyzlX17nElAY+9yjekeY" +
       "2LHKMu3Tiql7HKMsnP+d2F\n7pfSuD1GKyCvN6Uv/DIPRhkp7CHMUtA1GCd3" +
       "Hu0etyfEG9LIbLERQE83/zEpelbKpaqdGHLJM8mN\nd9j94KYjdk4eWeimJ+" +
       "UB80zI+h6edqv4CDPrQc8Oo6QSwmJCzl2NMTqfhyJ6dGmOzIyA4WzD9WYV\n" +
       "k6i1k6TsnPYX6cJVmuCxnlprYwb5YRUdunnLFkCqD5yJnVWjYjnuKFqt1ai5" +
       "YQs2deQt5RacGEi6\nHLXVt/WdMmaoXRyrh0w3IS8cHey+16ZDDF4RPxqsFJ" +
       "+8w3lN4la2487Sb0vvxuEVO04keRPsQitC\ndwrdldNvubsjgRH0Iylo46Dy" +
       "Y7mD9xbs+SQI+1KpCEcJWE2f6oW9ACwGNDBkELmmcHbULAP3AWqY\nKKJpk+" +
       "cD9P7G72LhQTNGA4WzzC08gwoMDmkJCtGiwPEED3cwkT6QC4g4pCuUc1LZqt" +
       "SayKP02uZS\nYzcDXS9Lb6XGCd/vfE32qQvfjG2GRD53806UKdvQ0ggOpJx4" +
       "4mbqotEFlHVTqeXBtifdYjUjCYN7\n0CqwyIBGbEUAkjUTMe3U5ZTLgka2Rp" +
       "4QPQvRh9Bo+lPMBDT4YEWU2AKBoyOEUSRb3sFcyzyki9DR\nHc9GEqZouOTD" +
       "Z7ujfb3luLZ4KMpwPpuEuOQVvhjMkLvWrMvj8qZrBFJjydIgzLE11MsBdUq1" +
       "iy2V\nf9Q3ufJZkkEMMcH9na1XeRP5ezmz9yYNnf3zubbOsuhVBpmdaX5tzV" +
       "VStXHSCo836JpIMkiNFQGC\n12spIxlmb8mg4yjH7irt8uu9EG1AGWwNB3O+" +
       "mF2NAM6FCTWPdUYa9HFWIQmSO1a24cTerpNph7St\nZbbv7G8JY+F7D4euZv" +
       "OI551XSDBwu+N8rtkapwkzWporgh15OoXLh9+sBqek/iXRb8khMAffs+xp\n" +
       "c5wGSMoTviXi174ODBg9OlJX7zRdBwx6754WlL+egiLX4D0UVyf/+iAxO7Na" +
       "HRu8Kz8lrCD7ew2P\nqMM1xsZkFAS18hxZRdpDgDohPMXmLh5IC0JXpxkg2D" +
       "a107EtPdt18vMxpzWMY7tD0INcR8wtdrtT\niykyTspfBtc7Nnx2PWUdBJss" +
       "BkqLe0d3lqjjvgZZnich8oO/Q62ISIY5q5er78x4gd2E2zU8n2nr\n6twGw8" +
       "MdsHkU1xzYe+yp6JmxxGxURO06Gq67OETuxL15NDcD1FD/pCy5CN+DdOoloV" +
       "F0NgaJo9Zx\ngXfB+Gjvj1MvN/XBNqYCLIUHcOYO2nEzphmKwstOl0GK5g6q" +
       "pmpnfOCOEm1Gl5k4ZxAFqVsFcTyz\n/D0/8drjci4V/aiBzD4GttE3VoX01o" +
       "xY0DuWnDBMmLeDblc5wUE6n5tY0sfpIVTrOa7vzb0Z0HY5\nb9riOXHrqTeo" +
       "9iFIQI3tcAiQOrnZWnWovDwPycMOK2llOz4/SCWl8c6wJdB0bl4WV46bxPEy" +
       "Ioge\nWwLQlElkygG5suzhQruxZZ6Vrqud8sDjRwjnZIZuKryFLVzf4Wp2Ur" +
       "VrGjg9Hbm+M0KCUVzrOnq0\nbhEKplmUmNqJoyyx7dlGYahuJ0P19PH+uHsR" +
       "n0g8ICewxuVhIOySSkcTeUVdihz5UqiZYskh8Dqv\nWBgT0LXxlwlXcVOOdD" +
       "wFQoIYAQw6jXmMjyRDM5Z7yaOHd5i3WJp6u3D0AMjzyuKsaRkwdnFax+0B\n" +
       "cyqzwuJDiS2nNXsoNwSoBB2ZKk6Jya7oROVgOMmwv5rZMbomsI67BznbJXIu" +
       "J20kkejVFPr52oVY\nT+jKNUTFooRmXh2kGLBN6vyoS6cTs0riTqs5KidGs0" +
       "+1KlbjhMs5RjZp1e2ATjuiyGlEYjjWwGQ9\ns8mtYjyCpM6R+cjcQsVvbG1E" +
       "7iwCKr3C2JY1VguWqeI6OQwDTD14Y0ApTXr0sONb5Trc51M/SOZt\nWnXxrN" +
       "s9m9Adsimcufm8Zk+cWauEIqbbV4tICYI5nuYTis8MVJ7I6GoSwI2KOkqIdu" +
       "2jZNn5rkeS\nJ8iyBCGuInpdEydkYW3ar7AlXW51J9PTGHvcIm1p+KizmcN4" +
       "Skoc9pwWUPrTevUMCXvs4hKa8i1n\n0E6tp92aWbKtK1EkAqfr7boQvjUZZ5" +
       "kgRx8lclwOaojWNjfg0HPaMdGwb0/xUdxni19Ux8OuCDuQ\nmI9oJ/QSZjBl" +
       "WtsP+bwaDN7XICYHY27qdrM/ibxcQYuNBet2YNOhWmgtVVPHwLRST2ngonOz" +
       "tOOV\ni3lJGA+fbiCO38NGk3iQBvHZxS/WPHnRHI2IAhp+ZFxdXZMHdPB6MG" +
       "txsNnKZSSnWLI0cdBg/YXa\nMaHuPFgpVA1xQQmW1WHsonINDct0Rl2wkL5l" +
       "kEHAYaxnIEPCbjadDiUUEDOpupNxE3gOqdnlPjtk\neNsZPnAxuZZ1gsuKN0" +
       "pIu5u/kJTcnbRSdRXOuFE4apWwS08oVGIloLNmEaePAsV7OA4Pjk6rQcW7\n" +
       "AaPnO8je0Fg+wySQfeUMy9mexpbNYSf8kmURQG1oUvDkbmvLF3IxqnYsM2Wr" +
       "Aw60RR3dhb3ZpCtr\nfFLQ1Y6gvYAmkn1cchbpMDoPXiayELuqZsSpzQxIs5" +
       "OCK8cra1q+VE2cIMfXo7fVbhO+aS5cmUL7\n3JTA6MWuv9zC/Tn7yF/xkmxe" +
       "ksvxGIdiUmbuI7xblqTVe9c1YU4c/Zs6Oe5dlTMeCRw/t4fNOZ1k\n+h7vdz" +
       "pAXbmLe/cnfVz9ihLNzAXWEXblpT2t4ZHo1IM4hStAVo506TnlVEleQrp7H5" +
       "9aiIntWsU7\n3GOvGkTu+rlbJHxYWhWjC3h2bdC7Xw2l9rIqYSlMZeHTy0k5" +
       "CJTOa+1mlSVwSm8p8p33hdmEmaTi\naoRIHvdoB/P8fKLdU/Zg1STMy1zOTw" +
       "9iiJtWPzaHaN+NjUZjk6IJo4ATE5gZj2CfYX0t7xVUELeC\nMxnRq8p5RsDv" +
       "jkeUvA6dfNsycaI73+62l6AEenGhzjULTggy3hgKReubA1ko+7xhl+rCXq9l" +
       "gRja\n8X4wjqyx3o99wV2MHRvXsMFognNbm+gec5kgIONeRCN/Im70cLCPl0" +
       "DfKujUHq2FgyM3P3XjkhYY\nj5LQCCbX8B5oy/Fc9jq0qwzWveDOBMEje6hJ" +
       "JHLAvSTvDaapw3OznDXEp/KTT8uLBwoy6ZnS3fYl\nYSYDqCQf/kHv0q0sFs" +
       "6mTo27uzNP91mhw3notOxaMATpOKUYZ0MwAbCB3PYn1OXQEQ9SgJPrrYQs\n" +
       "KI5KGMFIZi3BUVCTRj3OzL3UArsSQDl6Eo9yrhrjWaZyfjLJLiTILNtrfnRK" +
       "JgMCZnKlC6S6Rc3M\nrYOigAzR6Ok6kP0tYGZI84PhFpDcrq7n8yJlTgILUr" +
       "TVv4dLKGUcVC98WjRNZNFElHNIpaAw7yi5\nOpN37U5Y1uWMkchFxVz0jm4/" +
       "O/Ri0+GWunskSGzJm8SuKzteQXUREW/LN9KghmEe7kUUrWnRnBB0\nBVc9yC" +
       "XkXpzvy0hm/DwXvJoiF6xiiLzG1l1ezX6umJn8QNgK3TS0F4fS80DMOu8RFs" +
       "RcINCoGyM0\nq8bwEcyGmq2SJMLCg4DqkR2VI8o9zizFtcxx1xzKSyUChMnv" +
       "mS6+RNFRBaULc7APOBF3OgLCDhA0\n8+Fp0VuEMe7ZpJZx9wj9qT0+pPjOJY" +
       "22v7J3mTvt2K2iVVqvrLriMQhVw5vXLfLlh8oH1+sllonb\ndIQ0oPJVsqIf" +
       "6w1rkMo7NY8q99UoNYwtDKv4WjHg+Jh3oirdBbkyk4Y1tmooRFbtLolhqB/P" +
       "rXUj\n0AOnHLKIxhIx2kpu2JN65OJn0hZCDiRP+WI+rJLBCetFquRd0MCVAd" +
       "9c4qom5+LOYud1XFXVLy1p\nP5NlACVmruJAH/gICWWOJOb3FBqMPm+xAxOJ" +
       "tSDOJ/i28jnfLrvaAypY6BNLjOGKjZEBuPnAXS6V\nyBWBOcbqKEX845bWk+" +
       "SVuCUtYoLWDJ1Ms0bVckCaEjfvm74dB+sM7Sywd+SDjWgTQVoEJcfwXHOJ\n" +
       "TmMoHpex70LmlqkrLjFZUNCwp/bKdDzmODgJHBIFX5kI4uZ2oD1P6cCdcELX" +
       "3LJhFlMYq8ilM1gE\nFwWlae8OewBTsCpy1KUWnPYUwDnccppSK4LDZCtqSS" +
       "TAdIHGQFuxwNpWwx1SWg5JEAKpiud5qRoc\noSgkAu6QPl84kCov2cHZw+Pm" +
       "esBGW5CwmopHxNb0QsA146dKG08pBPZeidTormm9cLwCUBvmheyY\nfazT6M" +
       "E/qNVK0PqB0Kb0QeExS6HCViZeHWjOtSgJwSXEzOgquZce6EaEW271hQR3cn" +
       "Feam+MH76J\nlVVxSlIYOjdZa1K8HoRFdIbLdric9eZ2IIURMm02j2qoOxHM" +
       "NZVtHJKugOGQBx1f9unuuM8Ir7lQ\n14AwjbVoFDVffCajj8mRvFc41vhXXM" +
       "R9yVlDq+N6zAcYQcAuiwBRoRRyIybQxS259+i5oncPraCU\nfr9HBlCj/ce1" +
       "LRkDvREIY2cijEdM4590nCOxPX7CrWyc6qd1JQcGJN3WM4NOMDU7SveFHZeP" +
       "HRyn\nF+kG4GmUU1o/RTQrAjihrgRsOOpCbSkM7WFTUK8suAoxLLYei8cUhu" +
       "M3LPZWkzvGNy4H88GAcXXX\nqPZxs2jDowBGHu/WuOYisg4okfZO2J4LVNxf" +
       "azXw4gDp2Fv7YML0XBQ2fK601diYNRfxfr8XziUB\nk11BWhd3YhL+YPXOA7" +
       "7B7IW+0AcbuLIwugfRYUvtbuA9d+mz+ki2JDMmSKyG8KAQzPvEn3TjPrhu\n" +
       "CtkHytjdaoYP57w86VdnoHw1B/CMuDySPOMaBnSlvAN8nqacrOoVz+9ixOqv" +
       "/GnEI2E/F/cZiU7r\nhFRQmDyCfreVlJery7vK44D7vDh7ZYsq86oqD7cmzo" +
       "uJl7gqHbszJ9+7G6TOEYGdzOwhwCRbN/Z2\nbriS6usQh5mu7qDGNBFI20Ic" +
       "qkOATyu3vZLLasVSxdlsNnWBoEcbsGf2hgKRgM0VMuXJeGEElpJZ\nMxXKg5" +
       "oXDX+1RFTdldPpVhAcm8x7h/XNeHg8vFE4IpSmXzL3zBM6jIKLZa1AF2l4gs" +
       "UxG0STlGAX\nDmOndBqzgWtjiTmsubqbFGU/ydgUR/ypt/1cr8LcvVycSe0k" +
       "O75f50N4ILyQe9jkmVD5ammM0wwf\nWDvjWh8m9GpSSCCZtP0lQnYLU+CAlL" +
       "qWJjHXZobzvSqN9yYTjJaMMpmkcCc4uTlHAKamH2CgWc2B\nJ5GFTITFveFz" +
       "3Nyv6y2aNu0adoB/Ot9K8bwlfuS6Ob24nR/pkasOF3EFYBgfxFwr4yDZ6xod" +
       "xUY3\n6hFzwcnQRTMBJsiP/YV09+6fSHnqSNsTY5vEEtxiKWtWAPfBx3kujn" +
       "UJzCwuP/B9WbrafisKZpTS\nc45FtJsIcWVgnywmPRPwzd6FCOE+tJDZ38f9" +
       "M4KjhN7LSwpnrWXYwppUbK7dSskGSPK4QrNpYrEv\nTV482xePATzHvxNi7V" +
       "+0m+TBuzEkbRaHsPNV6FuXaLFr3/ZYvdWJB2whCDs1NOOe47ynnQi3gICU\n" +
       "sORb7SIzpabXBtaxhxoWBOvo+LXaDaOuBUdCvA2+uSeNLjVa3qeTK71eJhc9" +
       "gzTjm3W6HqL1RqTU\njHXcDEnWDRDj3vSbpjNjNEgR6IQcDpedWqosksnlXU" +
       "TieXREt5wAB7uX5MHto+RUqZqjjxM9JiPR\noI8HGkEweCA4JUhnRS+tKr64" +
       "l2R08uzg4TuMxatzMLcWtdTcQVPQOGRisCFgjwByqqy5+fksWrlA\ni7IzEe" +
       "FIKBkRBLbUGTfOTkkDEBLADxg2C3YHBVDKjUt+Shxm9lVUC70pKnC4IzCLGn" +
       "uufHnGTlFG\nRc7teLmWC2tgXg0m4JVD4wILSrTi13Q+RcmOwc8HP8uLashV" +
       "qYwOMSa4oEdMbgZeqi0DJJ/PwEo5\nXTanBjPi3Oa+C94L4Nhq7rrnoVSd2d" +
       "mbPP0y7ZqVlgtgXJfw3tDHhbCcuTG7jKNmu+BDz2unCUyM\nhWoQoohN5FCJ" +
       "96hT5JCEsKg6Dc4VgbU94lTuVvXveqKOR7cTuMelm2O8KckseGj63D+0xzSD" +
       "Ti/c\nE4S8GAxN1zD7qNyrInEjlVh6AajIoQv2D+qc0f6QotTO2DLboo8yr+" +
       "rhW3sU57MQpnxLdidB70LN\nPdjx6WC1bIagxiPUfCrK7MtilKvrXNrzeBbE" +
       "+shPzRmPvHTHbcUADlBHsvHlws1k+ua1ArqFAJhm\n8nXw5QRUuXOILgkUdi" +
       "RQkxkE8pnWXwGK7+CMflyGBxDHyx2sLzuyBFESbz548PI7xT+ANL1Pre72\n" +
       "VFdqooSHhFCOYEnIS534VH+3sZXIqILJ3QiWR71Uhf20k26Cts/3EOshEHzu" +
       "21Sm9XDUsetkzexW\nuDTtOloAEhfnA1cJZtgZa8rlCn6/+lIwTVsEdMAl60" +
       "4HjvN2/c1fVSTKD+IhaSNqXL1lQcgQPZfY\njA7AaCxb9lVKo5LTxG2lR/ww" +
       "6EF91ImbO8mnxNSwEzgxI4tANb1b2iMY35VCQq8kYIJptN49gACa\nsK6AZk" +
       "6cXj1Vp8oEQRpIECxCKxQ7yT7LXpu4ZBnLJZrog+J39/7/YTmvD2el9ldiPy" +
       "E1cSBTiRru\nxTwzwF0SnPY84xwhNrDALNLFuzYM9vyfsGYch1tWotOlWuHS" +
       "VrySuSEJgco7muSNkeKezqcTTh0u\nrGxRB4bV5mY9GOCLTPpI1Wk4CUb+dK" +
       "7Hh3s/1eEFiu5DKCE6cVR36GHLVMBI0Q26rmqLPBemsB0a\nFMtHpTpDDQcE" +
       "B1iBMgSq6EU43hVriCi4SgEKTIMCXXxzMBpdDMyMnXbONul4Vs/+OjJVAnZB" +
       "huzv\nXHZMYfeoHmT4PI1scDrHoepvbjcEne2dBN+PJZ+Ta9niAhqsJoFro3" +
       "TjdovfA1M1IfqNqyoc6b0C\nxXVNDzIizYNwcNW4pMa8r9fjftyOISxXMsfA" +
       "id6qhALy0rutisuizaXFr82Ow2lcnUevQDBq1qba\nLVtBPiMtRojbmUiGrZ" +
       "d+Db7hT/x1vBXNACtUKXzL0LuZrMBO0cExQIpjRZWdZodmQvQ+NBF95erN\n" +
       "WouWIK7+kajucLuXzjGvHYjINrV1XQgFCO5nT2kwJLp1eXrvj7vqmF5vKB/n" +
       "TdfPdz3UyypwZAId\nPe68ZTzTww0WpmswN2iZ5OpqHS3fIp3Rb4KstUYZrX" +
       "Y2YMWWsa/zujOL+7Vetti5d/AT3TUzP/qL\nU3t7jD8zrmsc+Tpz9STIDcGQ" +
       "UsFmqqatu3WrmL1JMI7weLgXGAOtDzUzdifwcl7wrp7p9HR1tLTF\n/IfWQ2" +
       "3F0sxj891ZyTBBx5B3uwVd3zYCIMXmEo+9cgCOHoXlvdjJ9BWO7cjetcfCXq" +
       "N9g6MzkwQu\nJ2XQAfJuGFu4rcm5vmUacjzAR1RdJxhzJG0RGHt+CDkOc0pM" +
       "TL4PT5PRlEWzT3ZSqliS6cbZwKMs\nJ5HKybN1U6vCfYoizIxdmdtRQrdMYl" +
       "82J4DbIgVuWEQYXYki0CUOq70DHxYmX2EUvjM5Y8uPgfAq\nGrzaV/qVa7Vg" +
       "qysfRD+6Md+wyRF7KMS8zr6FtRcyytPqNK0ljMDoCPgY1FOVpzl7QfVOuy2D" +
       "mLji\ngpxI1p5mxNjKQx/m5AvFTYrkcOrF2G4MjVbr0JmUcLuJfXorv8dYvl" +
       "IZjV1t9DzhDgCIx2U3YtNp\nLf1eP/seafZwF3OOawujYupkXB32CDLspWjs" +
       "VZV3Gy3o4VoT6nWrVZQOi+2HyLU3Ohr6+FwB3q5F\nxJvksyrG3vtbgEqKM2" +
       "P2Fk9qJ64f7VnrjQKMnf12qlKPSRPi3HP/dm0AR2Mnc6zoPWjazqnUmLKq\n" +
       "dxDacXf6mrp+nDFuoAY3mge0/axcWWE5njPUx08FGN4DsCCrhSz0obFPqgKx" +
       "ao+JvgNsUZl1Cb/b\ntw6xU5l6K72j8LgSjsfsDc4YsWuhSEfAg2TDwM6ZDW" +
       "KqDNS2bdyYkhaKFJGiavYi1iK7DmwweKsZ\nIQdL9HDnHw2FZo48Bx7opTyk" +
       "HFrMdCtFXLs8IAuYRanGihRG7IW4ItGhR4GmsoETApAhNDVQQ49C\n5NRVaR" +
       "R9t7NDbARr9tEdgS2RU86ZoMyyirrJdI8Iwl/juxbyF+3AUCugRXFrLDbl5U" +
       "1WAULERzyN\n7yuInuo1pTl9h6YdSt7suHU0MzW4yzFu9tcN2ZnqI9DR0FVJ" +
       "ueMQOXtZm+XoJkWeLJsG0EB24IQS\nZthFmjCEek4U4rJTHAQ4Uw/FRQmcoT" +
       "AQdisC01a8bC3Mn701fdztrByve2PSrsT9JPMirK9CMLWZ\nHMS2uRdbh7L3" +
       "qnE7rDstrO+ZZfMe3pfn4xUj76MtH+htjbNHP7YczbcDMrl61qXlI0s4OsY6" +
       "rVSx\n5a70I6pSQ99ruEh6wSgp8s6QmUBxHdsLjTW2NFD31Pxh88no9sIyKW" +
       "wvzxnZP249u/bGFuqzI3oT\nQXGm921+9LXzopiz75xdN+rC3cAyZNDR6d1t" +
       "ARQec/RMHSntfhLJsYHUOymsJnM+uKEJAsH+rGCW\nda5tlLWtuxCG/GKzrn" +
       "QP5kCsPGrd3RoomIbucdz8EqXcOHyCLVzBw+FSIA3t8hlXwL14YlXpuvD3\n" +
       "c4K6/Hq8MvZRAg5VhVxdG70E4Kld45jfYRY6zyofXMEHVHQD7K4oHzr1AbIw" +
       "mE0H2cdhw0pk8+BM\nC9kaleqeLLdM71bZHPRcgqGkGO19n9L7k7OT1Fuz54" +
       "nqcdnq7xSC91rqLwxJDlY7dWYf6uCW1roM\ncCaG9QT5612SOcXVYjGOUiby" +
       "cd1WSJnsDUpDwJ1zERIaMCs605AzpfpAnSGsBzcxwNKXW2FeuWNX\nXBsaWg" +
       "5Jn7FZdrkpy0U3SztO6Xxk4FXyUPkQLiBc7Ao3sblISp02oLEytPLrfU0P8h" +
       "AEkJwuzKNf\nbsv+rNkMuAcQHQvj2nFdjyfYzuZKmF7VaDuDehAfx8Hd2T6o" +
       "7O/BibkCg5W0fLIGj+RBP7eJeyQe\nHmsZZQ/gnQWg7txH0VDtC/TMEFtZeS" +
       "sclqUTCuHR5HZ4+NTufOhZsVqRvPejzYboh88vFLGgjkxx\nd9ep+qKG6S3A" +
       "mSAUYyq7no5H3cj8dbVHpA0jEG9HShJzFY5xfPf8/PUPvvcV7VdePtr9oD3R" +
       "p3+u\n++6H1H/ok18Cv/3hV8bz8OprG4cnl3eeTRXeeTajeUd8vvvGp3H85W" +
       "/Sx+CH3mf2/EL4HT6N3usA\n9M05/uHn5VeGVz/ysdkX6vlpstc9G918CwZ/" +
       "5D0GP/gxBlt6mXwbk//oG1c363c7ELzOYP5m8vzp\n97uYPK/AR7qTfNAT5a" +
       "sf9ET5QDjPVhg/+mktnF7aYPzS7e986fd5f/EXn5+PP/lQw6vvHurmZ4vw\n" +
       "ERYf9k15nYn00ojh/WYi//rnvxp8VsR+6PXGKT+9Lf/j33TmO/7wZ9/5P/4K" +
       "+ld/+u3Xu5DsunAY\nu8r8WC+SH/1YR4xn44Hjk177Av+l+82vfoo03/tE/Y" +
       "0dYz7z4efs3/jg8gtvHvtaR5qf+sgH/EUR\nxl6Bd/FYhtVAz37YPLtLbUr8" +
       "hZfv2JXoQ4X5Ux/s6IvPVjMb+U96047+7PPyW75pq5tv+vLdtjlf\nfxrj15" +
       "/G+PVPQyFslD7pTSj+/Kej+Ij4iA8v31R8z1///W8f2e/YqH7Sm5D9J98+sl" +
       "/4dUX21MWf\n3OiffdKbkP2lXwdd/GbC/PCEvxh82LPiZcJrUL+20W/d6A89" +
       "6U1Q/5tPgfo7vwnUz74M++xrUP8h\nROwnoZ8/xWt7RRp8omPDo37v4cdF/t" +
       "s2+uNPetM+/vo/gsg/9zLscx83/2+5mY8IvwirdLP4504+\nTYOfXUD+gye9" +
       "CfT/8h1b+BficHBCr3vT2T+X/+GN/sMnvWn5v/MdL/9d2/KfqntffK9xyp97" +
       "0pvW\n/7XveP3Pb+tT7/U7esPqz+42f/5Jb1r9H3zHq3/p3dXfbbDzaRieju" +
       "I/f9IbMLz1+V8XDB9003m+\n/oU3YHh6gP/6SW/C8D3fMYavbBiMj7X1+TQg" +
       "v2Wjv/qkNwH5/u8YyJc/eiDfDMc/ttFff9KbcPzI\nd4zjB94XyLcF5tnn6e" +
       "8+6U1gfurTwXzEq37PtzSUexjVXfi6q/3Cva6L0Ks+RW///pPehOq3//qg\n" +
       "+pwXDS9d8976mTfL5a0vPulNCI6/PgheOncRL7J5kwF/7V1deetrT3oTjDf0" +
       "wPv1D58fwv3aEy7+\nFBrvVe/s39mfIehNuJ/hEtgwo096E27yHyFcvv3hsG" +
       "98W2h3aS+GXvNefHrjGf/oBvAff9KbQIrf\nsSF+1/Be7fVptrdFiLfIJ71p" +
       "feNb6tjL71s18AMf1GIfLTW/pVGG7ei9VF1vlM2W77zFPOlN2H7X\np2N77Z" +
       "zeun6HOLYc4i31SW/C4X/nZ5RsxTVZB280wPc80f8/xjAQxmZ/FsX285Tkg8" +
       "4fw59OQGVR\nJAhjc0MhxW7gBaZTUH2B3xGgiIgHYWyOqCCYWPHkWbQGOvjg" +
       "TrDJWFwhDVSYBMLYXFFPRVcgzsjE\n0bBmTAVhbM5oo6IzYAdqYusFWoDVMU" +
       "BSALojeskoY9HrBj2YaYRLW/bElJRIHPEGcqsl0J2lIIzN\nrdPo61YOoFt9" +
       "ccUuyLHmQIdWgjA2x86nr2PZgI51wZ4G2BkgjWtGBvDZt/XgwTaX1OSckHyv" +
       "xLJE\n14qS+GQ90EiRgkYy7OjmdNjRzZpWxuYmVgrAsrc4s7A0vyRVA3Kqsg" +
       "Ko/6sAGobIzCvLz051SU1D\nOvJaQ1OhuiQjs1gPdZxCQ9O6VhNxGDTjCorL" +
       "JAlcLgApvYcWEHyQuhwaEIzSJAaEkZkBSkBADplV\ngIYHtJEKDhLYSGJ+mk" +
       "Y0+FxaBdSjoasTc5NqwSp9dBQgCiBjoxCJ/Fg3YPhlpilo5Ctkwo1SAAUh\n" +
       "UBxMKyQr2CpogEXyrRUgo5JoloSCDg0uSs0BsYtD8jWS9bCMfOuAHQGMFqgR" +
       "acAqNhU1kjZQ2AqD\nNT+QTgmFDQTjj1s+1KAsRK/5QceNM0B674wgNkhQCc" +
       "nheyhvPjJuJ+RIMZAjkxOLS1BDFls+5GCA\njKVC82E8icnP0NgYNR9CjrBG" +
       "zoiZJaCMpxAdG6yAEoXgQcipFLV/YBZg5CtoLIjgioWzOGIBZQKB\nPEeBZz" +
       "CAvVicEzCkuPMa7dwph+5OLFM9pDj1Pu2cKoXuVLRJJVKc+YKOIYpl+grURA" +
       "RlRdCh/hIY\n17xALiNJVrqUoHG4QPAk5Nhv2IUhbD4MHGmlOTnI588jsdkK" +
       "ilLTMsEeZYOcRg/2NeNnSD4BH9cO\ntBxEgfsrnyDS34DNNMj0JeP3AkjhAg" +
       "BNkUiihmYAAA==");
}
